// TODO Redo the create background stuff so that it handles timing better, especially when no image directory is set, but change at interval is
// TODO If the config form is showing when the screens are changed, reset the small image.

using System;
using System.Collections;
using System.Collections.Specialized;

using System.Runtime.InteropServices;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;

using Microsoft.Win32;
using System.IO;

using TrayRunnerFramework;
using System.Runtime.Serialization.Formatters.Binary;

namespace BackgroundCreator {
   [TrayRunnerFramework.TrayProgram("BackgroundCreator", "BackgroundCreator.App.ico",  Description="Test description")]
   [Serializable]
   /// <summary>
   /// Class to determine all the information necessary to create the image, as well as create the image.
   /// </summary>
   public class BackgroundInformation : MarshalByRefObject, TrayRunnerFramework.ITrayProgram {
      Random ranGen = new Random();

      private TrayMenuItem[] _trayMenu;
      public TrayMenuItem[] TrayMenu {
         get { return _trayMenu; }
      }

      ArrayList monitors;

      Properties _props;
      public Properties Props {
         get { return _props; } 
      }

      private MonitorInformation[] _monitorArray;
      /// <summary>
      /// List of all the monitors found.
      /// </summary>
      public MonitorInformation[] Monitors {
         get { return _monitorArray; }
      }

      Bitmap _smallVirtualScreenImage;
      /// <summary>
      /// An image of the screen scaled down to fit the config form.
      /// </summary>
      public Bitmap SmallVirtualScreenImage {
         get { return _smallVirtualScreenImage; }
      }

      private bool _changeAtInterval = false;
      /// <summary>
      /// Should the background images be changed at a set interval?
      /// </summary>
      public bool ChangeAtInterval {
         get { return _changeAtInterval; }
         set { 
            _props.ChangeAtInterval = value;
            if (_changeAtInterval != value) {
               _changeAtInterval= value; 
               if (_changeAtInterval== true) {
                  changeAtIntervalTimer.Start();
                  _timeOfLastChange = DateTime.Now;;
                  _timeOfNextChange = DateTime.Now.AddMilliseconds(ChangeAtIntervalMilliseconds);
               } else {
                  changeAtIntervalTimer.Stop();
               }
            }
         }
      }

      DateTime timeNextChange = DateTime.MinValue;

      private int _changeAtIntervalInterval = 30;
      /// <summary>
      /// Interval (in minutes) between background changes (if ChangeAtInterval is true)
      /// Defaults to 30 minutes.
      /// </summary>
      public int ChangeAtIntervalInterval {
         get { return _changeAtIntervalInterval; }
         set { 
            _props.ChangeAtIntervalMinutes = value;
            if (_changeAtIntervalInterval != value) {
               _changeAtIntervalInterval = value; 
               _timeOfLastChange = DateTime.Now;
               _changeAtIntervalMilliseconds = _changeAtIntervalInterval * 60 * 1000;
               _timeOfNextChange = DateTime.Now.AddMilliseconds(ChangeAtIntervalMilliseconds);

               changeAtIntervalTimer.Start();
            }
         }
      }

      private int _changeAtIntervalMilliseconds = 30 * 60 * 1000;
      public int ChangeAtIntervalMilliseconds {
         get { return _changeAtIntervalMilliseconds; }
      }

      private DateTime _timeOfLastChange;
      /// <summary>
      /// Time of last change. To be used to calculate time remaining.
      /// </summary>
      public DateTime TimeOfLastChange {
         get { return _timeOfLastChange; }
      }

      private DateTime _timeOfNextChange;
      /// <summary>
      /// Time of next change. To be used to calculate time remaining.
      /// </summary>
      public DateTime TimeOfNextChange {
         get { return _timeOfNextChange; }
      }

      // Objects for the interval imter
      private System.Timers.Timer changeAtIntervalTimer;

      // Config form variable so we can ensure only one version is seen
      ConfigForm cf = null;

      // Info to update the registry and where to store the image.
      private RegistryKey desktopKey;
      private String currentWallpaperLoc = "";

      // Info for the small image that is displayed on the config form.
      float smallScale;
      int smallWidthOffset;
      int smallHeightOffset;

      /// <summary>
      /// Constructor.
      /// </summary>
      /// <param name="forCreateOnly">If true, it does as little as possible to setup the image
      /// and it does not save any new config data.</param>
      public BackgroundInformation() {
         // Create the non-default close & config menu items
         _trayMenu = new TrayMenuItem[] {
                                           new TrayMenuItem("Config", new EventHandler(onConfig)),
                                           new TrayMenuItem("Change Background", new EventHandler(onChangeBackground))
                                        };

         try {
            _props = (Properties)SaveProperties.LoadObject("BackgroundCreator.options");
         } catch {
            _props = null;
         } // Just toss any exceptions so we can rebuild the properties...
            if (_props == null) _props = new Properties();

         if (_props.ViewedImages == null) _props.ViewedImages = new SortedList();

         _changeAtInterval = _props.ChangeAtInterval;
         _changeAtIntervalInterval = _props.ChangeAtIntervalMinutes;

         Microsoft.Win32.SystemEvents.DisplaySettingsChanged += new EventHandler(SystemEvents_DisplaySettingsChanged);
         
         // Send the startup to a separate thread to work around the stupid WIndows2000 OpenFileDialog bug with EnumDisplayMonitors
         System.Threading.Thread t = new System.Threading.Thread(new System.Threading.ThreadStart(doStartupThread));
         t.Start();
//         t.Join();
      }

      // This is really obnoxious, but it gets around a bizzarre bug in Windows 2000 with the OpenFileDialog
      private void doStartupThread() {
         try {
            // Setup the directory. Let's just store it in the users LocalApplicationData.
            String wallPaperDir = System.Environment.GetFolderPath(
               System.Environment.SpecialFolder.LocalApplicationData);

            currentWallpaperLoc = System.IO.Path.Combine(wallPaperDir, "CreatedWallpaper.bmp");
         
            // Make sure that it's tiled. If the current background isn't, all we can do is request they tile it...
            RegistryKey currentUser = Registry.CurrentUser;
            desktopKey = currentUser.OpenSubKey(@"Control Panel\Desktop", true);
            int tiledSetting = Convert.ToInt32(desktopKey.GetValue("TileWallpaper", 0));
            if (tiledSetting == 0) {
               MessageBox.Show("Please set the current wallpaper type to 'tiled'");
            }

            // Setup the timer but don't start it...
            _changeAtIntervalMilliseconds = _changeAtIntervalInterval * 1000 * 60;
            changeAtIntervalTimer = new System.Timers.Timer(500);
            changeAtIntervalTimer.Elapsed += new System.Timers.ElapsedEventHandler(changeAtIntervalElapsed);

            getMonitorInformation();

            if (_props.ImageDirectory != null && _props.ImageDirectory != "") {
               DirectoryInfo di = new DirectoryInfo(_props.ImageDirectory);
               if (!di.Exists) {
                  MessageBox.Show("Images directory does not exist. Please set a new one", "Invalid Image Directory", MessageBoxButtons.OK,
                     MessageBoxIcon.Exclamation);
                  _props.ImageDirectory = null;
               }
            }
               
            
            if (_changeAtInterval) {
               // If it'll be changed at a regular interval, might as well do it on startup
               _timeOfNextChange = DateTime.Now;
               createBackgroundImage();
               changeAtIntervalTimer.Start();
            }
         } catch (Exception ex) {
            MessageBox.Show("Exception:" + ex);
         }
      }

      private void getMonitorInformation() {
         // Create the small virtual screen image that will be used with the config form.
         // Scales so that it will center
         _smallVirtualScreenImage = new Bitmap(240, 120);

         Graphics g = Graphics.FromImage(_smallVirtualScreenImage);
         g.FillRectangle(Brushes.Black, 0, 0, 
            _smallVirtualScreenImage.Width, 
            _smallVirtualScreenImage.Height);

         smallScale = 0;
         smallWidthOffset = 0;
         smallHeightOffset = 0;

         float widthScale = (float)240 / (float)SystemInformation.VirtualScreen.Width;
         float heightScale = (float)120 / (float)SystemInformation.VirtualScreen.Height;
         if (widthScale > heightScale) {
            smallScale = heightScale;
            int width = Convert.ToInt32(SystemInformation.VirtualScreen.Width * smallScale);
            smallWidthOffset = Convert.ToInt32( (240 - width) / 2);
         } else {
            smallScale = widthScale;
            int height = Convert.ToInt32(SystemInformation.VirtualScreen.Height * smallScale);
            smallHeightOffset = Convert.ToInt32( (120 - height) / 2);
         }

         monitorCount = 0;
         MonitorInformation.resetMonitorCount();
			
         // Enumerate the monitors. Store item in an array list for reorganizing later.
         monitors = new ArrayList();
         foreach (Screen scr in Screen.AllScreens) {
            LayoutMonitor(scr);
         }

         // Set up the properties

         // Move the MonitorInformation objects into the monitor array so they are handy
         // Also sets the background type for each monitor.
         _monitorArray = new MonitorInformation[monitors.Count];
         for (int i = 0; i < monitors.Count; i++) {
            _monitorArray[i] = (MonitorInformation)monitors[i];

            String name = String.Format("Monitor{0}", i);
            if (_props.BackgroundTypePerMonitor.Length < i -1) {
               _monitorArray[i].BackgroundType= BackgroundType.None;
               continue;
            }
            if (_props.BackgroundTypePerMonitor.Length > i) {
               _monitorArray[i].BackgroundType = _props.BackgroundTypePerMonitor[i];
               if (_monitorArray[i].BackgroundType == BackgroundType.SpecificImage) {
                  _monitorArray[i].SpecificImageFile = _props.SpecificImagePerMonitor[i];
               }
            }
         }		
      }

      // If the user chooses the config menu option, or double clicks the icon,
      // show the config form.
      public void onConfig(Object sender, EventArgs e) {
         if (cf == null || cf.Visible == false) {
            cf = new ConfigForm(this);
            cf.Closed += new EventHandler(cf_Closed);
            cf.Show();
         }
      }

      public void onChangeBackground(Object sender, EventArgs e) {
         // Disable the changeAtInterval timer while we do this, or it'll fire off another change
         // immediately
         changeAtIntervalTimer.Enabled = false;
         _timeOfNextChange = DateTime.Now;
         createBackgroundImage();
         changeAtIntervalTimer.Enabled = true;
      }

      // Checks once a minute to see if enough time has elapsed to set the new time.
      // Done this way to allow the user to change the minutes and have it stay accurate.
      private void changeAtIntervalElapsed(Object sender, System.Timers.ElapsedEventArgs e) {
         if (_timeOfNextChange < DateTime.Now) {
            changeAtIntervalTimer.Enabled = false;
            createBackgroundImage();
            changeAtIntervalTimer.Enabled = true;
         }
      }

      // Find a random image. Returns empty string if no images can be found.
      private String getRandomImage(ArrayList images) {
         if (images.Count > 0) {
            int next = ranGen.Next(images.Count);

            return (String)images[next];
         } else {
            return "";
         }
      }

      // Finds all the images in the current and subdirectories
      // Returns an array list that the program can then Random into.
      private ArrayList getImages(String _imageDir) {
         ArrayList al = new ArrayList();

         String[] files = Directory.GetFiles(_imageDir);
         foreach (String file in files) {
            FileInfo fi = new FileInfo(file);
            switch (fi.Extension.ToLower()) {
               case ".bmp":
               case ".jpg":
               case ".jpeg":
               case ".gif":
               case ".png":
                  al.Add(file);
                  break;
            }
         }
         String[] dirs = Directory.GetDirectories(_imageDir);
         foreach (String dir in dirs) {
            ArrayList subAl = getImages(dir);
            al.AddRange(subAl);
         }
         return al;
      }

      private bool IsForegroundFullScreen() {
         IntPtr handle = GetForegroundWindow();
         RECT r;
            
         bool result = GetWindowRect(handle, out r);
         // If the Foreground window takes up the entire full screen. (Games, Slideshow, IE)
         // We don't want to change the background because it might not be seen, and or might take up too many 
         // resources
         if ( (r.left == 0 && r.top == 0)
            || (r.left == -1 && r.top == -1) ) {

            Size s = SystemInformation.PrimaryMonitorSize;
            if (r.bottom == s.Height && r.right == s.Width) return true;
            // The -1,-1 offsets are due to behavior seen by IE on the developer's machine
            if (r.bottom == s.Height + 1 && r.right == s.Width + 1) return true;
         }
         return false;
      }
      
      /// <summary>
      /// Sets up the information to create the image. 
      /// </summary>
      internal void createBackgroundImage() {
         if (_timeOfNextChange > DateTime.Now) return;

         // If the screen saver is active, just leave the current image in place
         // If we're in a remote desktop connect, leave the current image, otherwise it get's all messed up.
         // If the current app is in fullscreen mode, leave the current to avoid using System Resources
         bool isScreenSaverRunning = true;
         SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, 0, ref isScreenSaverRunning, 0);
         if (isScreenSaverRunning || GetSystemMetrics(SM_REMOTESESSION) == 1 || IsForegroundFullScreen()) {
				// Set it to change within the minute so that it changes shortly after we unlock the console
            _timeOfNextChange = _timeOfNextChange.AddMinutes(1);
            return;
         }

         lock (this) {
            String[] imageFiles = new String[_monitorArray.Length];
            ArrayList images = new ArrayList();

            for (int i = 0; i < _monitorArray.Length; i++) {
               // Sets each monitor to the specified type.
               MonitorInformation mi = _monitorArray[i];
               switch (mi.BackgroundType) {
                  case BackgroundType.None:
                     imageFiles[i] = "";
                     break;

                  case BackgroundType.SpecificImage:
                     if (_monitorArray[i].SpecificImageFile== null ||
                        _monitorArray[i].SpecificImageFile == "") {
                        MessageBox.Show("Not all Monitors setup");
                        return;
                     }
                     imageFiles[i] = _monitorArray[i].SpecificImageFile;
                     images.Add(imageFiles[i]);
                     break;
               }
            }

            for (int i = 0; i < imageFiles.Length; i++) {
               MonitorInformation mi = _monitorArray[i];

               ArrayList al = getImages(_props.ImageDirectory);
					
               if (mi.BackgroundType == BackgroundType.Random) {
                  if (_props.ImageDirectory == null) {
                     MessageBox.Show("You must select an image directory!");
                     return;
                  }
                  String fn = "";
                  int j;
                  for (j = 0; j < al.Count * 10; j++) { 
                     // Try really hard to get a new image. As the image directory fills up
                     // this could get more difficult, so keep trying for several time the amount of
                     // images
                     fn = getRandomImage(al);
                     if (!_props.ViewedImages.ContainsValue(new ImageInformation(fn)) ) break;
                  }

                  System.Threading.Thread.Sleep(100);
                  _props.ViewedImages.Add(DateTime.Now, new ImageInformation(fn));

                  if (_props.ViewedImages.Count >= al.Count) {
                     // Shear off all but the last 10 % of the images
                     // TODO Change back to .9
                     // _props.ViewedImages.RemoveRange(0, Convert.ToInt32(_props.ViewedImages.Count * .7));
                     /*
							int count = Convert.ToInt32(_props.ViewedImageHistory.Count * .7);
                     int removedCount = 0;
                     foreach (DateTime key in _props.ViewedImageHistory.Keys) {
                        _props.ViewedImageHistory.Remove(key);
                        removedCount++;
                        if (removedCount > count) break;
                     }
							*/
							// TODO Find a way to make this work again; Make it more intelligent leaving a little history
							_props.ViewedImages.Clear();
                     
                     //for (int k = 0; k < count; k++) {
                     //    _props.ViewedImageHistory.RemoveAt(0);
                     //}
                  }

                  imageFiles[i] = fn;
                  if (fn != "") {
                     images.Add(fn);
                  }
               }
            }

            for (int i = 0; i < imageFiles.Length; i++) {
               _monitorArray[i].CurrentImageFile = imageFiles[i];
            }

            createBackgroundImage(imageFiles);
            GC.Collect(); 
            // Force a GC to bring the memory back down. These iamges can be rather large.

				SaveCurrentList();
         }
      }

      private void createBackgroundImage(String[] imageFiles) {
         // Restart the timer
         _timeOfLastChange = DateTime.Now;
         _timeOfNextChange = DateTime.Now.AddMilliseconds(ChangeAtIntervalMilliseconds);

         // Make sure the files are good, or set the screen to blank.
         ArrayList monitorImages = new ArrayList();
         foreach (String s in imageFiles) {
            if (s == null || s == "") {
               monitorImages.Add("");
            } else {
               try {
                  Image img = Image.FromFile(s);
                  monitorImages.Add(img);
               } catch (System.IO.IOException) {
                  MessageBox.Show("Problem loading up the image: " + s + "\nPlease check your config settings");
               }
            }
         }

         Rectangle imageSize = SystemInformation.VirtualScreen;
         Bitmap newImage = new Bitmap(imageSize.Width, imageSize.Height);

         Graphics g = Graphics.FromImage(newImage);

         foreach (MonitorInformation mi in _monitorArray) {
            Object o = monitorImages[mi.MonitorId];
            Image currentImage = null;
            if (o is Image) {
               currentImage = (Image)monitorImages[mi.MonitorId];
            }
            
            // Create a bitmap the size of the current monitor, draw the image centered on that
            // bitmap, then place that bitmap onto the "final" created bitmap.
            // Done this way for ease of tiling across the monitor boundaries if the 
            // image must be split in two.
            Bitmap toPlaceImage = new Bitmap(mi.MonitorDimensions.Width, mi.MonitorDimensions.Height);
            Graphics toPlaceGraphics = Graphics.FromImage(toPlaceImage);
            toPlaceGraphics.FillRectangle(SystemBrushes.Desktop, 0, 0, toPlaceImage.Width, toPlaceImage.Height);
            if (currentImage != null) {
               drawImageCentered(toPlaceGraphics, currentImage, 
                  new Rectangle(0, 0, toPlaceImage.Width, toPlaceImage.Height));
            }

            Rectangle rect = mi.MonitorDimensions;
            Rectangle drawRect;
            if (rect.Top == 0 && rect.Left == 0) {
               // If it's the primary, we just draw the image
               drawRect = rect;
            } else {
               // Crosses the 0,0 boundary
               if (rect.Left < 0 && rect.Width > -rect.Left || rect.Top < 0 && rect.Height > -rect.Top) {
                  bool crossesTop = false;
                  if (rect.Top < 0 && rect.Bottom > 0) {
                     crossesTop = true;
                  }
                  int x = rect.Left;
                  int y = rect.Top;
                  if (x < 0) x = imageSize.Width + x;
                  if (y < 0) y = imageSize.Height + y;

                  int width = rect.Width;
                  int height = rect.Height;

                  Rectangle imageOffset;
                  if (crossesTop) {
                     // If it crossed the top, we draw the top of the image on the bottom of the tiled
                     // image, and the rest of the bottom of the image along the top.
                     drawRect = new Rectangle(x, 0, width, rect.Bottom);
                     imageOffset = new Rectangle(0, -rect.Top, width, height +  rect.Top);
                  } else {
                     // If it crosses the left, then the right of the image is on the left,
                     // and the left side on the right...
                     drawRect = new Rectangle(0, y, rect.Right, height);
                     imageOffset = new Rectangle(-rect.Left, 0, width + rect.Left, height);
                  }
                  g.DrawImage(toPlaceImage, drawRect, imageOffset, GraphicsUnit.Pixel);

                  // Since we've drawn the one portion on the one side, now we draw the rest on
                  // the other side in the normal manner.

                  drawRect = new Rectangle(x, y, width, height);
               } else {
                  // If it's not the primary, or doesn't cross the 0,0 boundary
                  // Just draw it in the appropraite location to tile to the secondary.
                  int x = rect.Left;
                  int y = rect.Top;
                  if (x < 0) x = imageSize.Width + x;
                  if (y < 0) y = imageSize.Height + y;
                  drawRect = new Rectangle(x, y, rect.Width, rect.Height);
               }
            }
            g.DrawImage(toPlaceImage, drawRect);
         }
         newImage.Save(currentWallpaperLoc, System.Drawing.Imaging.ImageFormat.Bmp);

         newImage.Dispose();

         // Change the wallpaper, then store it's location just in case
         SystemParametersInfo(SPI_SETDESKWALLPAPAER, 0, currentWallpaperLoc, 0);
         desktopKey.SetValue("Wallpaper", currentWallpaperLoc);

      }

      int monitorCount;

		private void adjustRectIfDockedWindow(String name, ref Rectangle monitorRect) {
			Process[] procs =  System.Diagnostics.Process.GetProcessesByName("dsidebar");
			foreach (Process p in procs) {
				RECT r;
				GetWindowRect(p.MainWindowHandle, out r);

				if (r.top != r.bottom) {
					if (r.top == monitorRect.Top && r.bottom == monitorRect.Bottom) {
						if (r.left == monitorRect.Left) {
							monitorRect.Size = new Size(monitorRect.Width - (r.right - r.left), monitorRect.Height);
							monitorRect.Location = new Point(r.right, monitorRect.Top);
						} else if (r.right == monitorRect.Right) {
							monitorRect.Size = new Size(monitorRect.Width - (r.right - r.left), monitorRect.Height);
						}
					}

					/*
					if (r.Left == monitorRect.Left && r.Right == monitorRect.Bottom) {
						if (r.Top == monitorRect.Top) {
	//						monitorRect.Top = r.Bottom;
						} else if (r.Bottom == monitorRect.Bottom) {
	//						monitorRect.Bottom = r.Top;
						}
					}
					*/
				}
			}
		}

      // Stores the monitor information and also lays out the config screen image with the small monitor representations.
      private bool LayoutMonitor(Screen scr) {
         Rectangle monitorRect = scr.Bounds;
			
         bool isPrimary = scr.Primary;

         if (isPrimary && _props.AccountForTaskBar) {
            monitorRect = SystemInformation.WorkingArea;
         }
			
         adjustRectIfDockedWindow("dsidebar", ref monitorRect);

         MonitorInformation mi = new MonitorInformation(monitorRect, isPrimary);
         if (mi.IsPrimary) {
            monitors.Insert(0, mi);
         } else {
            monitors.Add(mi);
         }
         Size size = monitorRect.Size;
         Rectangle imageSize = SystemInformation.VirtualScreen;

         Rectangle r = new Rectangle(monitorRect.Left - imageSize.Left, monitorRect.Top - imageSize.Top, monitorRect.Width, monitorRect.Height);

         Rectangle smallRect = new Rectangle(Convert.ToInt32(r.X * smallScale) + smallWidthOffset + 2, 
            Convert.ToInt32(r.Y * smallScale) + smallHeightOffset + 2,
            Convert.ToInt32(r.Width * smallScale) -4, Convert.ToInt32(r.Height * smallScale) -4);

         Graphics smallG = Graphics.FromImage(_smallVirtualScreenImage);

         smallG.DrawRectangle(new Pen(Brushes.White, 2), smallRect);
         smallG.FillRectangle(Brushes.DarkBlue, smallRect);

         int displayMonitorId = mi.MonitorId + 1;

         Font f = new Font(FontFamily.GenericSansSerif, 30, FontStyle.Bold);
         SizeF sf = smallG.MeasureString(displayMonitorId.ToString(), f);

         int x = Convert.ToInt32((smallRect.Width) / 2 - (sf.Width / 2)) + smallRect.X;
         int y = Convert.ToInt32((smallRect.Height) / 2 - (sf.Height/ 2)) + smallRect.Y;

         smallG.DrawString(displayMonitorId.ToString(), f, Brushes.White, x, y);

         mi.ConfigRectangle = smallRect;

         monitorCount++;

         return true;
      }
      
      // Draws the image centered on the given rectangle. Will scale down the image 
      // appropriately if it's too big in either dimension.
      private void drawImageCentered(Graphics g, Image img, Rectangle monitorRect) {
         float hPercent = (float)monitorRect.Height / (float)img.Height;
         float wPercent = (float)monitorRect.Width / (float)img.Width;

         int height = monitorRect.Height;
         int width = monitorRect.Width;

         int newX = 0;
         int newY = 0;

         if (hPercent > 1 && wPercent > 1 && _props.ScaleImageUp == false) {
            height = img.Height;
            width = img.Width;
            newX = (int)((float)(monitorRect.Width - width) / 2);
            newY = (int)((float)(monitorRect.Height - height) / 2);
         } else {
            if (hPercent < wPercent) {
               width = (int)((float)img.Width* (float)hPercent);
               height = (int)((float)img.Height * (float)hPercent);
               newX = (int)((float)(monitorRect.Width - width) / 2);
            } else {
               width = (int)((float)img.Width * (float)wPercent);
               height = (int)((float)img.Height * (float)wPercent);
               newY = (int)((float)(monitorRect.Height - height) / 2);
            } 
         }

         Rectangle r = new Rectangle(newX, newY, width, height);

         g.DrawImage(img, r);
      }

      /// <summary>
      ///  Save the config data when we close the config form or the program.
      ///  Uses functions in the ConfigReaders dll to easily build the config file
      ///  if none, exists, or update the existing
      /// </summary>
		public void OnShutdown() {
			SaveCurrentList();
		}

		private void SaveCurrentList() {
         _props.BackgroundTypePerMonitor = new BackgroundType[_monitorArray.Length];
         _props.SpecificImagePerMonitor = new String[_monitorArray.Length];
         for (int i = 0; i < _monitorArray.Length; i++) {
            _props.BackgroundTypePerMonitor[i] = _monitorArray[i].BackgroundType;
            _props.SpecificImagePerMonitor[i] = _monitorArray[i].SpecificImageFile;
         }

         SaveProperties.SaveObject(_props, "BackgroundCreator.options" );
      }

      // Interop code to set the background and determine the monitor and system information information
      uint SPI_SETDESKWALLPAPAER = 0x014;
      uint SPI_GETSCREENSAVERRUNNING = 0x072;

      int SM_REMOTESESSION = 0x1000;

      [DllImport("user32")]
      public static extern bool SystemParametersInfo(
         uint uiAction, uint uiParam, String pvParam, uint fWinIni);

      [DllImport("user32")]
      public static extern bool SystemParametersInfo(
         uint uiAction, uint uiParam, ref bool pvParam, uint fWinIni);

      [DllImport("User32")]
      private static extern int GetSystemMetrics(int nIndex);

      [DllImport("User32")]
      private static extern IntPtr GetForegroundWindow();

      [DllImport("User32")]
      private static extern bool GetWindowRect(IntPtr hwnd, out RECT lpRect);

      [StructLayout(LayoutKind.Sequential)]
         struct RECT {
         public int left;
         public int top;
         public int right;
         public int bottom;
      }

      [DllImport("User32")]
      private static extern int GetWindowText(IntPtr handle, System.Text.StringBuilder sb, int maxCount);

      [DllImport("User32")]
      private static extern uint GetWindowModuleFileName(IntPtr handle, System.Text.StringBuilder sb, uint max);


      private void SystemEvents_DisplaySettingsChanged(object sender, EventArgs e) {
         getMonitorInformation();
      }

      private void cf_Closed(object sender, EventArgs e) {
         getMonitorInformation();
      }
   }
}

