using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using System.IO;

namespace BackgroundCreator {
   /// <summary>
   /// Configuration form to set various options. Also allows you to create the background image 
   /// at will.
   /// </summary>
   public class ConfigForm : System.Windows.Forms.Form {
      int currentMonitor = -1;
      private BackgroundInformation backInfo;

      Timer elapsedTimer;

      Bitmap offscreenBitmap;
      Graphics offscreenGraphics;
      
      
      private System.Windows.Forms.Label label1;
      private System.ComponentModel.IContainer components;
      private System.Windows.Forms.ComboBox monitorList;
      private System.Windows.Forms.Button selectImageFile;
      private System.Windows.Forms.TextBox fileName;

      private System.Windows.Forms.Button createBackground;
      private System.Windows.Forms.GroupBox groupBox1;
      private System.Windows.Forms.Panel monitorLayoutPanel;
      private System.Windows.Forms.PictureBox monitorLayout;
      private System.Windows.Forms.Panel panel1;
      private System.Windows.Forms.RadioButton radioSelectedImage;
      private System.Windows.Forms.RadioButton radioRandomImage;
      private System.Windows.Forms.RadioButton radioNoImage;
      private System.Windows.Forms.Label label2;
      private System.Windows.Forms.CheckBox changeAtInterval;

      private System.Windows.Forms.NumericUpDown changeAtIntervalInterval;
      private System.Windows.Forms.Panel panel2;
      private System.Windows.Forms.TextBox imageDirectory;
      private System.Windows.Forms.CheckBox scaleUp;
      private System.Windows.Forms.GroupBox optionsGroupBox;
      private System.Windows.Forms.CheckBox accountForTaskBar;
      private System.Windows.Forms.ProgressBar progressBar1;
      private System.Windows.Forms.Label label3;
      private System.Windows.Forms.ToolTip toolTip1;
      private System.Windows.Forms.Button viewImageHistory;
      private System.Windows.Forms.Button clearImageHistory;
      private System.Windows.Forms.Button setImageDir;

      /// <summary>
      /// Constructor.
      /// </summary>
      /// <param name="backInfo">Background information which has all the monitor information.</param>
      public ConfigForm(BackgroundInformation backInfo) {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         this.backInfo = backInfo;

         monitorList.Items.Add("Primary Monitor (1)");
         for (int i = 1; i < backInfo.Monitors.Length; i++) {
            monitorList.Items.Add(String.Format("Secondary Monitor ({0})", i + 1));
         }

         monitorList.SelectedItem = monitorList.Items[0];
         // Draw the small image so the user can click the image as well as select from the drop down.
         monitorLayout.Image = backInfo.SmallVirtualScreenImage;

         // Note: The order in which these happens is important since the check box can
         // change the interval
         changeAtIntervalInterval.Value = backInfo.ChangeAtIntervalInterval;
         changeAtInterval.Checked = backInfo.ChangeAtInterval;
         if (changeAtInterval.Checked) {
            changeAtIntervalInterval.Enabled = true;
         }

         scaleUp.Checked = backInfo.Props.ScaleImageUp;
         accountForTaskBar.Checked = backInfo.Props.AccountForTaskBar;

         // No matter what, let's highlight monitor 0
         monitorList.SelectedIndex = 0;
         monitorList_SelectedIndexChanged(this, EventArgs.Empty);

         imageDirectory.Text = backInfo.Props.ImageDirectory;
         
         elapsedTimer = new Timer();
         elapsedTimer.Interval = 500;
         elapsedTimer.Tick += new EventHandler(elapsedTimer_Tick);
         elapsedTimer.Start();

         // Fire it off now so we don't have a blink update
         elapsedTimer_Tick(this, EventArgs.Empty);

         offscreenBitmap = new Bitmap(this.Width, this.Height);
         offscreenGraphics = Graphics.FromImage(offscreenBitmap);
      }

      protected override void OnPaint(PaintEventArgs e) {
         base.OnPaint (new PaintEventArgs(offscreenGraphics, e.ClipRectangle));

         e.Graphics.DrawImage(offscreenBitmap, e.ClipRectangle, e.ClipRectangle, GraphicsUnit.Pixel);
      }


      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         this.components = new System.ComponentModel.Container();
         this.monitorList = new System.Windows.Forms.ComboBox();
         this.monitorLayoutPanel = new System.Windows.Forms.Panel();
         this.monitorLayout = new System.Windows.Forms.PictureBox();
         this.label1 = new System.Windows.Forms.Label();
         this.fileName = new System.Windows.Forms.TextBox();
         this.selectImageFile = new System.Windows.Forms.Button();
         this.createBackground = new System.Windows.Forms.Button();
         this.groupBox1 = new System.Windows.Forms.GroupBox();
         this.panel1 = new System.Windows.Forms.Panel();
         this.radioSelectedImage = new System.Windows.Forms.RadioButton();
         this.radioRandomImage = new System.Windows.Forms.RadioButton();
         this.radioNoImage = new System.Windows.Forms.RadioButton();
         this.changeAtInterval = new System.Windows.Forms.CheckBox();
         this.changeAtIntervalInterval = new System.Windows.Forms.NumericUpDown();
         this.label2 = new System.Windows.Forms.Label();
         this.setImageDir = new System.Windows.Forms.Button();
         this.panel2 = new System.Windows.Forms.Panel();
         this.imageDirectory = new System.Windows.Forms.TextBox();
         this.scaleUp = new System.Windows.Forms.CheckBox();
         this.optionsGroupBox = new System.Windows.Forms.GroupBox();
         this.accountForTaskBar = new System.Windows.Forms.CheckBox();
         this.progressBar1 = new System.Windows.Forms.ProgressBar();
         this.label3 = new System.Windows.Forms.Label();
         this.toolTip1 = new System.Windows.Forms.ToolTip(this.components);
         this.viewImageHistory = new System.Windows.Forms.Button();
         this.clearImageHistory = new System.Windows.Forms.Button();
         this.monitorLayoutPanel.SuspendLayout();
         this.groupBox1.SuspendLayout();
         ((System.ComponentModel.ISupportInitialize)(this.changeAtIntervalInterval)).BeginInit();
         this.optionsGroupBox.SuspendLayout();
         this.SuspendLayout();
         // 
         // monitorList
         // 
         this.monitorList.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.monitorList.Location = new System.Drawing.Point(312, 16);
         this.monitorList.Name = "monitorList";
         this.monitorList.Size = new System.Drawing.Size(192, 21);
         this.monitorList.TabIndex = 0;
         this.toolTip1.SetToolTip(this.monitorList, "Monitor the curret option list is for.");
         this.monitorList.SelectedIndexChanged += new System.EventHandler(this.monitorList_SelectedIndexChanged);
         // 
         // monitorLayoutPanel
         // 
         this.monitorLayoutPanel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
         this.monitorLayoutPanel.Controls.Add(this.monitorLayout);
         this.monitorLayoutPanel.Location = new System.Drawing.Point(8, 16);
         this.monitorLayoutPanel.Name = "monitorLayoutPanel";
         this.monitorLayoutPanel.Size = new System.Drawing.Size(242, 122);
         this.monitorLayoutPanel.TabIndex = 1;
         // 
         // monitorLayout
         // 
         this.monitorLayout.Dock = System.Windows.Forms.DockStyle.Fill;
         this.monitorLayout.Location = new System.Drawing.Point(0, 0);
         this.monitorLayout.Name = "monitorLayout";
         this.monitorLayout.Size = new System.Drawing.Size(240, 120);
         this.monitorLayout.SizeMode = System.Windows.Forms.PictureBoxSizeMode.StretchImage;
         this.monitorLayout.TabIndex = 1;
         this.monitorLayout.TabStop = false;
         this.toolTip1.SetToolTip(this.monitorLayout, "Displays the current monitor layout.");
         this.monitorLayout.DoubleClick += new System.EventHandler(this.monitorLayout_DoubleClick);
         this.monitorLayout.MouseDown += new System.Windows.Forms.MouseEventHandler(this.monitorLayout_MouseDown);
         // 
         // label1
         // 
         this.label1.Location = new System.Drawing.Point(264, 16);
         this.label1.Name = "label1";
         this.label1.Size = new System.Drawing.Size(48, 23);
         this.label1.TabIndex = 2;
         this.label1.Text = "Monitor";
         // 
         // fileName
         // 
         this.fileName.Enabled = false;
         this.fileName.Location = new System.Drawing.Point(264, 120);
         this.fileName.Name = "fileName";
         this.fileName.Size = new System.Drawing.Size(208, 20);
         this.fileName.TabIndex = 5;
         this.fileName.Text = "";
         this.toolTip1.SetToolTip(this.fileName, "If \"Selected Image\" is chosen, this is the actual image to display.");
         this.fileName.TextChanged += new System.EventHandler(this.fileName_TextChanged);
         // 
         // selectImageFile
         // 
         this.selectImageFile.Enabled = false;
         this.selectImageFile.Location = new System.Drawing.Point(480, 120);
         this.selectImageFile.Name = "selectImageFile";
         this.selectImageFile.Size = new System.Drawing.Size(24, 23);
         this.selectImageFile.TabIndex = 6;
         this.selectImageFile.Text = "...";
         this.toolTip1.SetToolTip(this.selectImageFile, "Browse foir the image.");
         this.selectImageFile.Click += new System.EventHandler(this.selectImageFile_Click);
         // 
         // createBackground
         // 
         this.createBackground.Location = new System.Drawing.Point(272, 160);
         this.createBackground.Name = "createBackground";
         this.createBackground.Size = new System.Drawing.Size(224, 23);
         this.createBackground.TabIndex = 7;
         this.createBackground.Text = "Create New Background";
         this.toolTip1.SetToolTip(this.createBackground, "Change the background now!");
         this.createBackground.Click += new System.EventHandler(this.createBackground_Click);
         // 
         // groupBox1
         // 
         this.groupBox1.Controls.Add(this.panel1);
         this.groupBox1.Controls.Add(this.radioSelectedImage);
         this.groupBox1.Controls.Add(this.radioRandomImage);
         this.groupBox1.Controls.Add(this.radioNoImage);
         this.groupBox1.Controls.Add(this.fileName);
         this.groupBox1.Controls.Add(this.selectImageFile);
         this.groupBox1.Controls.Add(this.label1);
         this.groupBox1.Controls.Add(this.monitorList);
         this.groupBox1.Controls.Add(this.monitorLayoutPanel);
         this.groupBox1.Location = new System.Drawing.Point(8, 0);
         this.groupBox1.Name = "groupBox1";
         this.groupBox1.Size = new System.Drawing.Size(512, 152);
         this.groupBox1.TabIndex = 8;
         this.groupBox1.TabStop = false;
         this.groupBox1.Text = "Display Image";
         // 
         // panel1
         // 
         this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
         this.panel1.Location = new System.Drawing.Point(264, 40);
         this.panel1.Name = "panel1";
         this.panel1.Size = new System.Drawing.Size(240, 4);
         this.panel1.TabIndex = 7;
         // 
         // radioSelectedImage
         // 
         this.radioSelectedImage.Location = new System.Drawing.Point(264, 96);
         this.radioSelectedImage.Name = "radioSelectedImage";
         this.radioSelectedImage.TabIndex = 2;
         this.radioSelectedImage.Text = "Selected Image";
         this.toolTip1.SetToolTip(this.radioSelectedImage, "Display a particular image on this monitor.");
         this.radioSelectedImage.CheckedChanged += new System.EventHandler(this.radioSelectedImage_CheckedChanged);
         // 
         // radioRandomImage
         // 
         this.radioRandomImage.Location = new System.Drawing.Point(264, 72);
         this.radioRandomImage.Name = "radioRandomImage";
         this.radioRandomImage.TabIndex = 1;
         this.radioRandomImage.Text = "Random Image";
         this.toolTip1.SetToolTip(this.radioRandomImage, "Display a random image on this monitor.");
         this.radioRandomImage.CheckedChanged += new System.EventHandler(this.radioRandomImage_CheckedChanged);
         // 
         // radioNoImage
         // 
         this.radioNoImage.Checked = true;
         this.radioNoImage.Location = new System.Drawing.Point(264, 48);
         this.radioNoImage.Name = "radioNoImage";
         this.radioNoImage.TabIndex = 0;
         this.radioNoImage.TabStop = true;
         this.radioNoImage.Text = "No Image";
         this.toolTip1.SetToolTip(this.radioNoImage, "Display no image on this background.");
         this.radioNoImage.CheckedChanged += new System.EventHandler(this.radioNoImage_CheckedChanged);
         // 
         // changeAtInterval
         // 
         this.changeAtInterval.Location = new System.Drawing.Point(8, 16);
         this.changeAtInterval.Name = "changeAtInterval";
         this.changeAtInterval.Size = new System.Drawing.Size(208, 24);
         this.changeAtInterval.TabIndex = 10;
         this.changeAtInterval.Text = "Change Background At Interval";
         this.toolTip1.SetToolTip(this.changeAtInterval, "Should be image be changed at an interval?");
         this.changeAtInterval.CheckedChanged += new System.EventHandler(this.changeAtInterval_CheckedChanged);
         // 
         // changeAtIntervalInterval
         // 
         this.changeAtIntervalInterval.Enabled = false;
         this.changeAtIntervalInterval.Location = new System.Drawing.Point(8, 40);
         this.changeAtIntervalInterval.Maximum = new System.Decimal(new int[] {
                                                                                 120,
                                                                                 0,
                                                                                 0,
                                                                                 0});
         this.changeAtIntervalInterval.Minimum = new System.Decimal(new int[] {
                                                                                 1,
                                                                                 0,
                                                                                 0,
                                                                                 0});
         this.changeAtIntervalInterval.Name = "changeAtIntervalInterval";
         this.changeAtIntervalInterval.Size = new System.Drawing.Size(40, 20);
         this.changeAtIntervalInterval.TabIndex = 11;
         this.changeAtIntervalInterval.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
         this.toolTip1.SetToolTip(this.changeAtIntervalInterval, "Interval (in minutes) between background changes.");
         this.changeAtIntervalInterval.Value = new System.Decimal(new int[] {
                                                                               30,
                                                                               0,
                                                                               0,
                                                                               0});
         this.changeAtIntervalInterval.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.changeAtIntervalInterval_KeyPress);
         this.changeAtIntervalInterval.ValueChanged += new System.EventHandler(this.changeInterval_ValueChanged);
         // 
         // label2
         // 
         this.label2.Location = new System.Drawing.Point(48, 40);
         this.label2.Name = "label2";
         this.label2.TabIndex = 12;
         this.label2.Text = "Minutes";
         // 
         // setImageDir
         // 
         this.setImageDir.Location = new System.Drawing.Point(8, 296);
         this.setImageDir.Name = "setImageDir";
         this.setImageDir.Size = new System.Drawing.Size(104, 23);
         this.setImageDir.TabIndex = 13;
         this.setImageDir.Text = "Set Image Dir";
         this.toolTip1.SetToolTip(this.setImageDir, "Sets the image directory for random images.");
         this.setImageDir.Click += new System.EventHandler(this.setImageDir_Click);
         // 
         // panel2
         // 
         this.panel2.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
         this.panel2.Location = new System.Drawing.Point(8, 72);
         this.panel2.Name = "panel2";
         this.panel2.Size = new System.Drawing.Size(208, 4);
         this.panel2.TabIndex = 14;
         // 
         // imageDirectory
         // 
         this.imageDirectory.Location = new System.Drawing.Point(120, 296);
         this.imageDirectory.Name = "imageDirectory";
         this.imageDirectory.ReadOnly = true;
         this.imageDirectory.Size = new System.Drawing.Size(392, 20);
         this.imageDirectory.TabIndex = 15;
         this.imageDirectory.Text = "";
         this.toolTip1.SetToolTip(this.imageDirectory, "Directory (and all subdirectories) containing the images for the random backgroun" +
            "ds.");
         // 
         // scaleUp
         // 
         this.scaleUp.Location = new System.Drawing.Point(8, 80);
         this.scaleUp.Name = "scaleUp";
         this.scaleUp.TabIndex = 16;
         this.scaleUp.Text = "Scale Image Up";
         this.toolTip1.SetToolTip(this.scaleUp, "Scales the image up to the maximum size available");
         this.scaleUp.CheckedChanged += new System.EventHandler(this.scaleUp_CheckedChanged);
         // 
         // optionsGroupBox
         // 
         this.optionsGroupBox.Controls.Add(this.accountForTaskBar);
         this.optionsGroupBox.Controls.Add(this.changeAtInterval);
         this.optionsGroupBox.Controls.Add(this.changeAtIntervalInterval);
         this.optionsGroupBox.Controls.Add(this.label2);
         this.optionsGroupBox.Controls.Add(this.panel2);
         this.optionsGroupBox.Controls.Add(this.scaleUp);
         this.optionsGroupBox.Location = new System.Drawing.Point(8, 152);
         this.optionsGroupBox.Name = "optionsGroupBox";
         this.optionsGroupBox.Size = new System.Drawing.Size(248, 136);
         this.optionsGroupBox.TabIndex = 17;
         this.optionsGroupBox.TabStop = false;
         this.optionsGroupBox.Text = "Options";
         // 
         // accountForTaskBar
         // 
         this.accountForTaskBar.Location = new System.Drawing.Point(8, 104);
         this.accountForTaskBar.Name = "accountForTaskBar";
         this.accountForTaskBar.Size = new System.Drawing.Size(144, 24);
         this.accountForTaskBar.TabIndex = 17;
         this.accountForTaskBar.Text = "Account For Task Bar";
         this.toolTip1.SetToolTip(this.accountForTaskBar, "Will account for the taskbar so that no portion of the image is underneath.");
         this.accountForTaskBar.CheckedChanged += new System.EventHandler(this.accountForTaskBar_CheckedChanged);
         // 
         // progressBar1
         // 
         this.progressBar1.Location = new System.Drawing.Point(264, 256);
         this.progressBar1.Name = "progressBar1";
         this.progressBar1.Size = new System.Drawing.Size(248, 23);
         this.progressBar1.TabIndex = 18;
         this.toolTip1.SetToolTip(this.progressBar1, "Time until the next change of the background.");
         // 
         // label3
         // 
         this.label3.Location = new System.Drawing.Point(264, 232);
         this.label3.Name = "label3";
         this.label3.Size = new System.Drawing.Size(208, 23);
         this.label3.TabIndex = 19;
         this.label3.Text = "Time Until Next Change";
         // 
         // viewImageHistory
         // 
         this.viewImageHistory.Location = new System.Drawing.Point(272, 192);
         this.viewImageHistory.Name = "viewImageHistory";
         this.viewImageHistory.Size = new System.Drawing.Size(96, 23);
         this.viewImageHistory.TabIndex = 20;
         this.viewImageHistory.Text = "Image History";
         this.viewImageHistory.Click += new System.EventHandler(this.viewImageHistory_Click);
         // 
         // clearImageHistory
         // 
         this.clearImageHistory.Location = new System.Drawing.Point(400, 192);
         this.clearImageHistory.Name = "clearImageHistory";
         this.clearImageHistory.Size = new System.Drawing.Size(96, 23);
         this.clearImageHistory.TabIndex = 21;
         this.clearImageHistory.Text = "Clear";
         this.clearImageHistory.Click += new System.EventHandler(this.clearImageHistory_Click);
         // 
         // ConfigForm
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(522, 328);
         this.Controls.Add(this.clearImageHistory);
         this.Controls.Add(this.viewImageHistory);
         this.Controls.Add(this.label3);
         this.Controls.Add(this.progressBar1);
         this.Controls.Add(this.optionsGroupBox);
         this.Controls.Add(this.imageDirectory);
         this.Controls.Add(this.setImageDir);
         this.Controls.Add(this.groupBox1);
         this.Controls.Add(this.createBackground);
         this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
         this.HelpButton = true;
         this.MaximizeBox = false;
         this.Name = "ConfigForm";
         this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
         this.Text = "Background Creator Config";
         this.Closed += new System.EventHandler(this.ConfigForm_Closed);
         this.VisibleChanged += new System.EventHandler(this.ConfigForm_VisibleChanged);
         this.monitorLayoutPanel.ResumeLayout(false);
         this.groupBox1.ResumeLayout(false);
         ((System.ComponentModel.ISupportInitialize)(this.changeAtIntervalInterval)).EndInit();
         this.optionsGroupBox.ResumeLayout(false);
         this.ResumeLayout(false);

      }
      #endregion

      // Most of the following should be self-evident.
      private void ConfigForm_Closed(object sender, System.EventArgs e) {
         this.Visible = false;
         // Calls on shutdown to save the data
         backInfo.OnShutdown();
      }

      private void selectImageFile_Click(object sender, System.EventArgs e) {
         OpenFileDialog ofd = new OpenFileDialog();
         ofd.Multiselect = false;
         ofd.InitialDirectory = backInfo.Props.ImageDirectory;
         if (ofd.ShowDialog() == DialogResult.OK) {
            fileName.Text = ofd.FileName;
         }
      }

      private void fileName_TextChanged(object sender, System.EventArgs e) {
         backInfo.Monitors[currentMonitor].SpecificImageFile = fileName.Text;
      }

      private void monitorList_SelectedIndexChanged(object sender, System.EventArgs e) {
         currentMonitor = monitorList.SelectedIndex;

         MonitorInformation mi = backInfo.Monitors[currentMonitor];

         Bitmap highlightedBitmap = (Bitmap)backInfo.SmallVirtualScreenImage.Clone();
         Graphics g = Graphics.FromImage(highlightedBitmap);
         Pen p = new Pen(Color.LightBlue, 4);
         g.DrawRectangle(p,
            mi.ConfigRectangle.X, mi.ConfigRectangle.Y,
            mi.ConfigRectangle.Width - 1,
            mi.ConfigRectangle.Height - 1);
         monitorLayout.Image = highlightedBitmap;

         fileName.Text = mi.SpecificImageFile;

         switch (mi.BackgroundType) {
            case BackgroundType.None:
               radioNoImage.Checked = true;
               break;
            case BackgroundType.Random:
               radioRandomImage.Checked = true;
               break;
            case BackgroundType.SpecificImage:
               radioSelectedImage.Checked = true;
               break;
         }
      }

      private void createBackground_Click(object sender, System.EventArgs e) {
         backInfo.onChangeBackground(this, EventArgs.Empty);
      }


      private void radioNoImage_CheckedChanged(object sender, System.EventArgs e) {
         backInfo.Monitors[currentMonitor].BackgroundType = BackgroundType.None;
      }

      private void radioRandomImage_CheckedChanged(object sender, System.EventArgs e) {
         backInfo.Monitors[currentMonitor].BackgroundType = BackgroundType.Random;
      }

      private void radioSelectedImage_CheckedChanged(object sender, System.EventArgs e) {
         selectImageFile.Enabled = radioSelectedImage.Checked;
         fileName.Enabled = radioSelectedImage.Checked;
         backInfo.Monitors[currentMonitor].BackgroundType = BackgroundType.SpecificImage;
      }

      private void changeAtInterval_CheckedChanged(object sender, System.EventArgs e) {
         changeAtIntervalInterval.Enabled = changeAtInterval.Checked;
         backInfo.ChangeAtInterval = changeAtInterval.Checked;
         backInfo.ChangeAtIntervalInterval= Convert.ToInt32(changeAtIntervalInterval.Value);
      }

      private void changeInterval_ValueChanged(object sender, System.EventArgs e) {
         backInfo.ChangeAtIntervalInterval = Convert.ToInt32(changeAtIntervalInterval.Value);
      }

      // Value changed only picks up up/down events, so this is to pickup the text change.
      private void changeAtIntervalInterval_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e) {
         backInfo.ChangeAtIntervalInterval = Convert.ToInt32(changeAtIntervalInterval.Value);
      }

      private void setImageDir_Click(object sender, System.EventArgs e) {
         FolderBrowserDialog fbd = new FolderBrowserDialog();
         fbd.ShowNewFolderButton = false;

         if (backInfo.Props.ImageDirectory != null || backInfo.Props.ImageDirectory != "") {
            fbd.SelectedPath = backInfo.Props.ImageDirectory;
         }
         if (fbd.ShowDialog(this) == DialogResult.OK) {
            imageDirectory.Text = fbd.SelectedPath;
            backInfo.Props.ImageDirectory = fbd.SelectedPath;
         }
      }

      private FileInfo contextMenuImage = null;
		private Point mouseClickLoc = new Point(0,0);
      // When the click on a portion of the monitor layout screen, change the selected info
      // to that monitor.
		private void monitorLayout_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
			mouseClickLoc = new Point(e.X, e.Y);
			foreach (MonitorInformation mi in backInfo.Monitors) {
				if (mi.ConfigRectangle.Contains(new Point(e.X, e.Y) ) ) {
					if (e.Button == MouseButtons.Left) {
						monitorList.SelectedIndex = mi.MonitorId;
					} else if (e.Button == MouseButtons.Right) {
						ContextMenu menu = new ContextMenu();
						FileInfo fi = new FileInfo(mi.CurrentImageFile);
                  contextMenuImage = fi;
                  menu.MenuItems.Add(new MenuItem("View Image", new EventHandler(viewImage)));
                  menu.MenuItems.Add(new MenuItem("Delete Image", new EventHandler(removeImage)));
						menu.Show(monitorLayout, new Point(e.X, e.Y));
					}
				}
			}
		}

		// TODO Ok, this could probably be done better than just spinning up the built in viewer
		private void viewImage(Object sender, EventArgs e) {
			System.Diagnostics.Process.Start(contextMenuImage.FullName);
		}

      private void removeImage(object sender, EventArgs e) {
         if (MessageBox.Show(this, String.Format("Are you sure you want to delete - {0}", contextMenuImage.Name), "Confirm Delete?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
            contextMenuImage.Delete();
         }
      }

      private void scaleUp_CheckedChanged(object sender, System.EventArgs e) {
         backInfo.Props.ScaleImageUp = scaleUp.Checked;      
      }

      private void elapsedTimer_Tick(object sender, EventArgs e) {
         if (backInfo.ChangeAtInterval) {
            int interval = backInfo.ChangeAtIntervalMilliseconds;
            TimeSpan diff =  backInfo.TimeOfNextChange - DateTime.Now;

            double percent = ((double)diff.TotalMilliseconds/ (double)interval) * 100;

            if (percent > 0) {
               if (100 - percent > 0) {
                  progressBar1.Value = Convert.ToInt32(100 - percent);
               } else {
                  progressBar1.Value = 0;
               }
            }
         } else {
            progressBar1.Value = 0;
         }
      }

      private void ConfigForm_VisibleChanged(object sender, System.EventArgs e) {
         elapsedTimer.Enabled = this.Visible;
      }

      private void accountForTaskBar_CheckedChanged(object sender, System.EventArgs e) {
         backInfo.Props.AccountForTaskBar = accountForTaskBar.Checked;
      }

		private void monitorLayout_DoubleClick(object sender, System.EventArgs e) {
			foreach (MonitorInformation mi in backInfo.Monitors) {
				if (mi.ConfigRectangle.Contains(mouseClickLoc) ) {
					System.Diagnostics.Process.Start(mi.CurrentImageFile);
				}
			}
		}

      private void viewImageHistory_Click(object sender, System.EventArgs e) {
         ImageHistory ih = new ImageHistory(backInfo.Props.ViewedImages);
         ih.ShowDialog();
      }

      private void clearImageHistory_Click(object sender, System.EventArgs e) {
         backInfo.Props.ViewedImages.Clear();
      }
   }
}
