using System;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using TrayRunnerFramework;

using Microsoft.Win32;

namespace DataGrab {
   [TrayRunnerFramework.TrayProgram("DataGrabber", "litening.ico")]
   [Serializable]
   /// <summary>
   /// Used to grab various pieces of data from the system for easy interaction.
   /// 
   /// Current grabs a URL from the clipboard, or blog data from the Windows Media Blogging Plugin.
   /// 
   /// </summary>
   public class DataGrabber : MarshalByRefObject, ITrayProgram {

      public DataGrabber() {
         System.Threading.Thread.CurrentThread.SetApartmentState(System.Threading.ApartmentState.STA);
			_trayMenu = new TrayMenuItem[]{
														new TrayMenuItem("Show Current Song", new EventHandler(songGrabbed)), 
														new TrayMenuItem("Paste Current Song", new EventHandler(typeSong)),
														new TrayMenuItem("Grab URL", new EventHandler(urlGrabbed))
			} ;
			
      }

		public void test(Object sender, EventArgs e) {
			MessageBox.Show("Test");
		}

      private TrayMenuItem[] _trayMenu = new TrayMenuItem[0];
      public TrayMenuItem[] TrayMenu {
         get { return _trayMenu; }
      }

		private RegistryKey getPlayingSong() {
			RegistryKey key = Registry.CurrentUser;
			RegistryKey subKey = key.OpenSubKey("Software\\Microsoft\\MediaPlayer\\CurrentMetadata");
			
			if (subKey != null && subKey.GetValue("Title") != null) {
				return subKey;
			} else {
				MessageBox.Show("No song playing", "No song playing", MessageBoxButtons.OK, MessageBoxIcon.Exclamation,
					MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
				return null;
			}
		}

      public void songGrabbed(Object sender, EventArgs e) {
			RegistryKey subKey = getPlayingSong();
			
			if (subKey != null) {
				MessageBox.Show(String.Format("Now Playing:\nAuthor:{0}\nAlbum:{1}\nSong:{2} ({3})", subKey.GetValue("Author"), subKey.GetValue("Album"), subKey.GetValue("Title"),
					subKey.GetValue("durationString")), "Now Playing", MessageBoxButtons.OK, MessageBoxIcon.Information,
					MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
			}
      }

		public void typeSong(Object sender, EventArgs e) {
			RegistryKey subKey = getPlayingSong();

			if (subKey != null) {
				String s = String.Format("Now Playing: {0} - {1} - {2}", subKey.GetValue("Author"), subKey.GetValue("Album"), subKey.GetValue("Title"),
					subKey.GetValue("durationString"));
				SendKeys.SendWait(s);
			}
		}

		private void showNoUrl() {
			MessageBox.Show("No URL to view!", "No Url!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation,
				MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
		}

		/// <summary>
		/// Grabs the first item on the clip board, and if it's a string, starts up a new IE session. Will trim line breaks
		/// and whitespace on wrapped URLS.
		/// </summary>
		public void urlGrabbed(Object sender, EventArgs e) {
			IDataObject ido = Clipboard.GetDataObject();
			try {
				String s = (String)ido.GetData(typeof(System.String));
				if (s == null) { showNoUrl(); return; }
				// This removes white space and any other common chars that may sneak in.
				s = s.Replace("\r", "");
				s = s.Replace("\n", "");
				s = s.Replace("\t", "");
				s = s.Replace(" ", "");
				s = s.Replace(">", "");
				s = s.Trim();

				if (s.EndsWith(".") ) {
					s = s.Substring(0, s.Length - 1);
				}

				if (Regex.Match(s, @"\w*://").Success) {
					System.Diagnostics.Process.Start(@"c:\Program Files\Internet Explorer\IExplore.exe", s);
				} else {
					showNoUrl();
				}
			} catch (Exception ex) {
				MessageBox.Show(ex.ToString(), "Exception!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation,
					MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
			}
		}

		public void OnShutdown() {
      }

      public void KeepAlive() {
      }
   }
}
