using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Text;


using System.IO;

namespace ShowDesktopIcons
{
	/// <summary>
	/// Summary description for DesktopList.
	/// </summary>
	public class DesktopList : System.Windows.Forms.Form
	{
      private System.Windows.Forms.ContextMenu contextMenu1;
      private System.Windows.Forms.MenuItem renameFile;
      private System.Windows.Forms.MenuItem deleteFile;
      private System.Windows.Forms.ListView listView1;
      private System.Windows.Forms.ColumnHeader columnHeader1;
      private System.Windows.Forms.ColumnHeader columnHeader2;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

      ImageList images = new ImageList();

      FileSystemWatcher _watcher;

      
      public DesktopList() {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         //
         // TODO: Add any constructor code after InitializeComponent call
         //

         // Grab an All Users Directory, then step back to get the Desktop
         DirectoryInfo allUsersDesktop = new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData));
         allUsersDesktop = new DirectoryInfo(Path.Combine(allUsersDesktop.Parent.FullName, "Desktop"));

         DirectoryInfo userDesktop = new DirectoryInfo(Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory));

         buildIconList(allUsersDesktop, userDesktop);

         _watcher = new FileSystemWatcher(userDesktop.FullName);
         _watcher.Changed += new FileSystemEventHandler(_watcher_Changed);
         _watcher.Created += new FileSystemEventHandler(_watcher_Created);
         _watcher.Deleted += new FileSystemEventHandler(_watcher_Deleted);
         _watcher.Renamed += new RenamedEventHandler(_watcher_Renamed);

         _watcher.EnableRaisingEvents = true;
      }		

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

      protected override void OnVisibleChanged(EventArgs e) {
         base.OnVisibleChanged (e);

         if (this.Visible == true) {
            if (listView1.SelectedItems.Count == 1) {
               listView1.SelectedItems[0].Focused = true;
               listView1.FocusedItem.EnsureVisible();
            } else {
               if (listView1.Items.Count > 0) {
                  listView1.Items[0].Selected = true;
                  listView1.Items[0].Focused = true;

                  this.Focus();
               }
            }

            SetForegroundWindow(this.Handle);
         }
      }


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
         this.contextMenu1 = new System.Windows.Forms.ContextMenu();
         this.renameFile = new System.Windows.Forms.MenuItem();
         this.deleteFile = new System.Windows.Forms.MenuItem();
         this.listView1 = new System.Windows.Forms.ListView();
         this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
         this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
         this.SuspendLayout();
         // 
         // contextMenu1
         // 
         this.contextMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                     this.renameFile,
                                                                                     this.deleteFile});
         // 
         // renameFile
         // 
         this.renameFile.Index = 0;
         this.renameFile.Text = "Rename";
         this.renameFile.Click += new System.EventHandler(this.renameFile_Click);
         // 
         // deleteFile
         // 
         this.deleteFile.Index = 1;
         this.deleteFile.Text = "Delete";
         this.deleteFile.Click += new System.EventHandler(this.deleteFile_Click);
         // 
         // listView1
         // 
         this.listView1.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
                                                                                    this.columnHeader1,
                                                                                    this.columnHeader2});
         this.listView1.Dock = System.Windows.Forms.DockStyle.Fill;
         this.listView1.Location = new System.Drawing.Point(0, 0);
         this.listView1.Name = "listView1";
         this.listView1.Size = new System.Drawing.Size(376, 270);
         this.listView1.TabIndex = 0;
         this.listView1.View = System.Windows.Forms.View.Details;
         this.listView1.ItemActivate += new System.EventHandler(this.listView1_ItemActivate);
         this.listView1.MouseUp += new System.Windows.Forms.MouseEventHandler(this.listView1_MouseUp);
         // 
         // columnHeader1
         // 
         this.columnHeader1.Text = "Filename";
         this.columnHeader1.Width = 200;
         // 
         // columnHeader2
         // 
         this.columnHeader2.Text = "Date Modified";
         this.columnHeader2.Width = 150;
         // 
         // DesktopList
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(376, 270);
         this.Controls.Add(this.listView1);
         this.Name = "DesktopList";
         this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
         this.Text = "DesktopList";
         this.TopMost = true;
         this.ResumeLayout(false);

      }
		#endregion

      #region Item activation
      private void listView1_ItemActivate(object sender, System.EventArgs e) {
         ListViewItem lvi = listView1.FocusedItem;
         FileInfo fi = (FileInfo)lvi.Tag;
         try {
            this.Close();
            System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(itemActivate), fi.FullName);
         } catch (Exception ex) {
            MessageBox.Show("Can't execute file:" + ex.Message);
         }
      }

      private void itemActivate(Object state) {
         String name = (String)state;
         System.Diagnostics.Process.Start(name);
      }
 
      private void renameFile_Click(object sender, System.EventArgs e) {
         System.Diagnostics.Trace.WriteLine("Selected:" + listView1.SelectedItems[0]);

         MessageBox.Show("Not yet implemented");
      }

      private void deleteFile_Click(object sender, System.EventArgs e) {
         if (MessageBox.Show(this, "Are you sure you want to delete this item?", "Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Stop, MessageBoxDefaultButton.Button2) == DialogResult.Yes) {
            FileInfo fi = (FileInfo)listView1.SelectedItems[0].Tag;

            fi.Delete();

            listView1.Items.Remove(listView1.SelectedItems[0]);
         }
      }

      private void listView1_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
         if (e.Button == MouseButtons.Right && listView1.SelectedIndices.Count == 1) {
            contextMenu1.Show(listView1, new Point(e.X, e.Y));
         }
      }
      #endregion

      #region Capture Files and Icons
      private ShellFileInfo GetFileInfo(FileInfo fi) {
         try {
            SHFILEINFO shfi = new SHFILEINFO();
         
            SHGetFileInfo(fi.FullName, 0, ref shfi, 352, SHGFI_DISPLAYNAME | SHGFI_ICON | SHGFI_SMALLICON);

            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < 260; i++) {
               if (shfi.szDisplayName[i] == 0) break;
               sb.Append((char)shfi.szDisplayName[i]);    
            }
            Icon ico = Icon.FromHandle(shfi.hIcon);
            
            Bitmap b = ico.ToBitmap();

            ShellFileInfo sfi = new ShellFileInfo();
            sfi.DisplayName = sb.ToString();
            sfi.SmallIcon = b;
            return sfi;
         } catch (Exception ex) {
            MessageBox.Show("EX:" + ex);
         }
         return null;
      }

      [DllImport("Shell32.dll")]
      private static extern IntPtr SHGetFileInfo(String pszPath, int dwFileSttributes,
         ref SHFILEINFO psfi, uint cbFileInfo, uint uFlags);

      const int SHGFI_ICON = 0x100;
      const int SHGFI_DISPLAYNAME = 0x200;
      const int SHGFI_TYPENAME = 0x400;
      const int SHGFI_ATTRIBUTES = 0x800;
      const int SHGFI_ICONLOCATION = 0x1000;
      const int SHGFI_SMALLICON = 0x000000001;
      const int SHGFI_LARGEICON = 0x000000000;

      internal class ShellFileInfo {
         internal String DisplayName;
         internal Bitmap SmallIcon;
      }

      [StructLayout(LayoutKind.Sequential)]
         internal struct SHFILEINFO {
         public IntPtr hIcon;
         public int iIcon;
         public int dwAttributes;
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=260)]
         public byte[] szDisplayName;
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=80)]
         public byte[] szTypeName;
      }

      [System.Runtime.InteropServices.DllImport("User32.dll")]
      private static extern bool SetForegroundWindow(IntPtr hwnd);
      
      private void buildIconList(params DirectoryInfo[] allDirs) {
         lock (this) {
            images.Images.Clear();
            SortedList sl = new SortedList();  
            foreach (DirectoryInfo di in allDirs) {
               FileInfo[] files = di.GetFiles();

               foreach (FileInfo fi in files) {
                  ShellFileInfo sfi = GetFileInfo(fi);
                  images.Images.Add(sfi.SmallIcon);

                  DesktopListViewItem lvi = new DesktopListViewItem(null, new String[] {sfi.DisplayName, fi.LastWriteTime.ToString()} );
                  lvi.Tag = fi;
                  lvi.ImageIndex = images.Images.Count - 1;

                  sl.Add(fi.Name, lvi);
               }

               foreach (DirectoryInfo subDi in di.GetDirectories()) {
                  
               }
            }

            listView1.SmallImageList = images;
            listView1.Items.Clear();

            foreach (ListViewItem lvi in sl.Values) {
               listView1.Items.Add(lvi);
            }
         }
      }
#endregion

      #region FileWatcher
      private void _watcher_Created(object sender, FileSystemEventArgs e) {
         System.Threading.Thread.Sleep(100);
         FileInfo fi = new FileInfo(e.FullPath);
         ShellFileInfo sfi = GetFileInfo(fi);
         images.Images.Add(sfi.SmallIcon);

         ListViewItem lvi = new ListViewItem(new String[] {sfi.DisplayName, fi.LastWriteTime.ToString()} );
         lvi.Tag = fi;
         lvi.ImageIndex = images.Images.Count - 1;

         listView1.Items.Add(lvi);
      }

      private void _watcher_Changed(object sender, FileSystemEventArgs e) {
      }

      private void _watcher_Deleted(object sender, FileSystemEventArgs e) {
         foreach (ListViewItem lvi in listView1.Items) {
            FileInfo fi = lvi.Tag as FileInfo;
            if (fi.FullName.ToLower() == e.FullPath.ToLower() ) {
               lvi.Remove();
            }
         }
      }

      private void _watcher_Renamed(object sender, RenamedEventArgs e) {
         FileInfo newFi = new FileInfo(e.FullPath);
         ShellFileInfo sfi = GetFileInfo(newFi);

         foreach (ListViewItem lvi in listView1.Items) {
            FileInfo fi = lvi.Tag as FileInfo;
            if (fi.FullName.ToLower() == e.OldFullPath.ToLower() ) {
               lvi.Text = sfi.DisplayName;
               lvi.Tag = newFi;
            }
         }
      }
      #endregion
	}
}
