using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using System.Diagnostics;
using System.Runtime.Remoting;
using System.Reflection;
using System.Security.Policy;
using System.IO;

using TrayRunnerFramework;

namespace TrayRunner
{
   public class ConfigForm : System.Windows.Forms.Form {
      private System.Windows.Forms.MainMenu mainMenu1;
      private System.Windows.Forms.MenuItem fileMenu;
      private System.Windows.Forms.MenuItem exitMenu;
      private System.Windows.Forms.TabControl tabControl;
      private System.Windows.Forms.TabPage pluginsTab;
      private System.Windows.Forms.ListView programListView;
      private System.Windows.Forms.TabPage hotKeyTab;
      private System.Windows.Forms.TreeView hotKeyTree;

      /*
       Find domain directory
       Create Domain
       Load Assemblys
       Find Class with proper type
       If Class
          Check if Class is "Active"
          If Active
             Instantiate
             Store Domain Information
          End If
       Else
          DestroyDomain
       End If
      
      --- 
      When program list Check/Uncheck instantiate/load domain/unload domain
      ===
      FileSystemWatcher on all directories
      If change other than config file And Domain is Active
         Unload Domain
         Reload Domain
      End If
      */

      Hashtable domains;
      Hashtable hotKeys;
      ArrayList pluginDirectories;


      private System.Windows.Forms.MenuItem menuItem2;
      private System.Windows.Forms.MenuItem loadNewPlugin;
      private System.Windows.Forms.TabPage pluginDirs;
      private TrayRunner.ConfigPanes.PluginDirectoriesForm pluginDirectoriesForm1;
      private System.Windows.Forms.ColumnHeader columnHeader1;
      private System.Windows.Forms.ColumnHeader columnHeader2;
		private System.Windows.Forms.ImageList imageList1;
		private System.ComponentModel.IContainer components;
      private MenuItem menuItem1;
      private MenuItem menuHelpAbout;
      TrayRunner.Win32.DummyWindowWithEvent dummyWindow;
      
      public ConfigForm(TrayRunner.Win32.DummyWindowWithEvent dummyWindow, 
         Hashtable domains, 
         Hashtable hotKeys,
         ArrayList pluginDirectories) {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         this.domains = domains;
         this.hotKeys = hotKeys;
         this.dummyWindow = dummyWindow;
         this.pluginDirectories = pluginDirectories;

         foreach (String domainName in domains.Keys) {
            this.TopMost = true;
            AppDomainInformation adi = (AppDomainInformation)domains[domainName];

            addDomainInfo(domainName, adi);
         }

         pluginDirectoriesForm1.PluginDirectories = pluginDirectories;
         pluginDirectoriesForm1.NewPluginDirectory += new NewPluginDirectoryEventHandler(newPluginDirectoryEvent);
      }

      private void newPluginDirectoryEvent(Object sender, NewPluginDirectoryEventArgs e) {
         if (NewPluginDirectory != null) {
            // Find all pre-existing domains.
            ArrayList al = new ArrayList();
            foreach (String domainName in domains.Keys) {
               al.Add(domainName);
            }

            NewPluginDirectory(sender, e);

            foreach (String domainName in domains.Keys) {
               // Add info for any new domains...
               if (!al.Contains(domainName) ) {
                  addDomainInfo(domainName, (AppDomainInformation)domains[domainName]);
               }
            }
         }
      }

      private void addDomainInfo(String domainName, AppDomainInformation adi) {
         ListViewItem lvi = new ListViewItem(domainName);
         lvi.Checked = adi.IsActive;
         lvi.Tag = adi;

         lvi.ImageIndex = 0;

         if (adi.Plugin.PluginIcon != null) {
            MemoryStream ms = new MemoryStream(adi.Plugin.PluginIcon);
            Image i = Image.FromStream(ms);
            imageList1.Images.Add(i);
         }
         lvi.ImageIndex = imageList1.Images.Count - 1;

         programListView.Items.Add(lvi);
      }

      private void addHotKeyList(AppDomainInformation adi) {
         TreeNode currentNode = new TreeNode(adi.Name);
         // TODO Figure out what the deal with the TrayMenu is
         if (adi.TrayMenu != null && adi.TrayMenu.Length > 0) {
            // Alphabatize the hot key tree
            if (hotKeyTree.Nodes.Count == 0) {
               hotKeyTree.Nodes.Add(currentNode);
            } else {
               if (currentNode.Text.CompareTo(hotKeyTree.Nodes[0].Text) < 0 ) {
                  hotKeyTree.Nodes.Insert(0, currentNode);
               } else if (currentNode.Text.CompareTo(hotKeyTree.Nodes[hotKeyTree.Nodes.Count-1].Text) > 0) {
                  hotKeyTree.Nodes.Add(currentNode);
               } else {
                  for (int i = 0; i < hotKeyTree.Nodes.Count - 1; i++) {
                     if (currentNode.Text.CompareTo(hotKeyTree.Nodes[i].Text) > 0 && 
                        currentNode.Text.CompareTo(hotKeyTree.Nodes[i + 1].Text) < 0) {
                        hotKeyTree.Nodes.Insert( i + 1, currentNode);
                        break;
                     }
                  }
               }
            }

            for (int i = 0; i < adi.TrayMenu.Length; i++) {
               TreeNode tn = new TreeNode(adi.TrayMenu[i].Name);
               tn.Tag = adi.TrayMenu[i];
               currentNode.Nodes.Add(tn);

               String id = String.Format("{0}/{1}", adi.Name, adi.TrayMenu[i].Name);
               if (hotKeys.Contains(id) ) {
                  SystemHotKey shk = (SystemHotKey)hotKeys[id];
                  tn.Text = String.Format("{0} - ({1})", tn.Text, shk.Key);
               }
            }
            currentNode.Expand();
         }
      }


      private void newMenu_Click(Object sender, EventArgs e) {
         MessageBox.Show("No menu items specified");
      }
         

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
         }
         base.Dispose( disposing );
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         this.components = new System.ComponentModel.Container();
         System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ConfigForm));
         this.mainMenu1 = new System.Windows.Forms.MainMenu(this.components);
         this.fileMenu = new System.Windows.Forms.MenuItem();
         this.loadNewPlugin = new System.Windows.Forms.MenuItem();
         this.menuItem2 = new System.Windows.Forms.MenuItem();
         this.exitMenu = new System.Windows.Forms.MenuItem();
         this.tabControl = new System.Windows.Forms.TabControl();
         this.pluginsTab = new System.Windows.Forms.TabPage();
         this.programListView = new System.Windows.Forms.ListView();
         this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
         this.columnHeader2 = new System.Windows.Forms.ColumnHeader();
         this.imageList1 = new System.Windows.Forms.ImageList(this.components);
         this.hotKeyTab = new System.Windows.Forms.TabPage();
         this.hotKeyTree = new System.Windows.Forms.TreeView();
         this.pluginDirs = new System.Windows.Forms.TabPage();
         this.pluginDirectoriesForm1 = new TrayRunner.ConfigPanes.PluginDirectoriesForm();
         this.menuItem1 = new System.Windows.Forms.MenuItem();
         this.menuHelpAbout = new System.Windows.Forms.MenuItem();
         this.tabControl.SuspendLayout();
         this.pluginsTab.SuspendLayout();
         this.hotKeyTab.SuspendLayout();
         this.pluginDirs.SuspendLayout();
         this.SuspendLayout();
         // 
         // mainMenu1
         // 
         this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.fileMenu,
            this.menuItem1});
         // 
         // fileMenu
         // 
         this.fileMenu.Index = 0;
         this.fileMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.loadNewPlugin,
            this.menuItem2,
            this.exitMenu});
         this.fileMenu.Text = "&File";
         // 
         // loadNewPlugin
         // 
         this.loadNewPlugin.Index = 0;
         this.loadNewPlugin.Text = "&Load New Plug-in";
         this.loadNewPlugin.Click += new System.EventHandler(this.loadNewPlugin_Click);
         // 
         // menuItem2
         // 
         this.menuItem2.Index = 1;
         this.menuItem2.Text = "-";
         // 
         // exitMenu
         // 
         this.exitMenu.Index = 2;
         this.exitMenu.Text = "E&xit";
         this.exitMenu.Click += new System.EventHandler(this.exitMenu_Click);
         // 
         // tabControl
         // 
         this.tabControl.Controls.Add(this.pluginsTab);
         this.tabControl.Controls.Add(this.hotKeyTab);
         this.tabControl.Controls.Add(this.pluginDirs);
         this.tabControl.Dock = System.Windows.Forms.DockStyle.Fill;
         this.tabControl.Location = new System.Drawing.Point(0, 0);
         this.tabControl.Name = "tabControl";
         this.tabControl.SelectedIndex = 0;
         this.tabControl.Size = new System.Drawing.Size(368, 276);
         this.tabControl.TabIndex = 0;
         // 
         // pluginsTab
         // 
         this.pluginsTab.Controls.Add(this.programListView);
         this.pluginsTab.Location = new System.Drawing.Point(4, 22);
         this.pluginsTab.Name = "pluginsTab";
         this.pluginsTab.Size = new System.Drawing.Size(360, 250);
         this.pluginsTab.TabIndex = 0;
         this.pluginsTab.Text = "Plug-ins";
         // 
         // programListView
         // 
         this.programListView.CheckBoxes = true;
         this.programListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.columnHeader1,
            this.columnHeader2});
         this.programListView.Dock = System.Windows.Forms.DockStyle.Fill;
         this.programListView.FullRowSelect = true;
         this.programListView.GridLines = true;
         this.programListView.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
         this.programListView.Location = new System.Drawing.Point(0, 0);
         this.programListView.MultiSelect = false;
         this.programListView.Name = "programListView";
         this.programListView.Size = new System.Drawing.Size(360, 250);
         this.programListView.SmallImageList = this.imageList1;
         this.programListView.Sorting = System.Windows.Forms.SortOrder.Ascending;
         this.programListView.TabIndex = 1;
         this.programListView.UseCompatibleStateImageBehavior = false;
         this.programListView.View = System.Windows.Forms.View.Details;
         this.programListView.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(this.programListView_ItemCheck);
         // 
         // columnHeader1
         // 
         this.columnHeader1.Text = "Name";
         this.columnHeader1.Width = 104;
         // 
         // columnHeader2
         // 
         this.columnHeader2.Text = "Description";
         this.columnHeader2.Width = 232;
         // 
         // imageList1
         // 
         this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
         this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
         this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
         // 
         // hotKeyTab
         // 
         this.hotKeyTab.Controls.Add(this.hotKeyTree);
         this.hotKeyTab.Location = new System.Drawing.Point(4, 22);
         this.hotKeyTab.Name = "hotKeyTab";
         this.hotKeyTab.Size = new System.Drawing.Size(360, 0);
         this.hotKeyTab.TabIndex = 1;
         this.hotKeyTab.Text = "Hot Keys";
         // 
         // hotKeyTree
         // 
         this.hotKeyTree.Dock = System.Windows.Forms.DockStyle.Fill;
         this.hotKeyTree.Location = new System.Drawing.Point(0, 0);
         this.hotKeyTree.Name = "hotKeyTree";
         this.hotKeyTree.Size = new System.Drawing.Size(360, 0);
         this.hotKeyTree.TabIndex = 0;
         this.hotKeyTree.DoubleClick += new System.EventHandler(this.hotKeyTree_DoubleClick);
         this.hotKeyTree.BeforeCollapse += new System.Windows.Forms.TreeViewCancelEventHandler(this.hotKeyTree_BeforeCollapse);
         this.hotKeyTree.MouseDown += new System.Windows.Forms.MouseEventHandler(this.hotKeyTree_MouseDown);
         // 
         // pluginDirs
         // 
         this.pluginDirs.Controls.Add(this.pluginDirectoriesForm1);
         this.pluginDirs.Location = new System.Drawing.Point(4, 22);
         this.pluginDirs.Name = "pluginDirs";
         this.pluginDirs.Size = new System.Drawing.Size(360, 0);
         this.pluginDirs.TabIndex = 2;
         this.pluginDirs.Text = "Plugin Directories";
         // 
         // pluginDirectoriesForm1
         // 
         this.pluginDirectoriesForm1.Dock = System.Windows.Forms.DockStyle.Fill;
         this.pluginDirectoriesForm1.Location = new System.Drawing.Point(0, 0);
         this.pluginDirectoriesForm1.Name = "pluginDirectoriesForm1";
         this.pluginDirectoriesForm1.PluginDirectories = null;
         this.pluginDirectoriesForm1.Size = new System.Drawing.Size(360, 0);
         this.pluginDirectoriesForm1.TabIndex = 0;
         // 
         // menuItem1
         // 
         this.menuItem1.Index = 1;
         this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuHelpAbout});
         this.menuItem1.Text = "&Help";
         // 
         // menuHelpAbout
         // 
         this.menuHelpAbout.Index = 0;
         this.menuHelpAbout.Text = "&About";
         this.menuHelpAbout.Click += new System.EventHandler(this.menuHelpAbout_Click);
         // 
         // ConfigForm
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(368, 276);
         this.Controls.Add(this.tabControl);
         this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow;
         this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
         this.Menu = this.mainMenu1;
         this.Name = "ConfigForm";
         this.Text = "TrayRunner Config";
         this.tabControl.ResumeLayout(false);
         this.pluginsTab.ResumeLayout(false);
         this.hotKeyTab.ResumeLayout(false);
         this.pluginDirs.ResumeLayout(false);
         this.ResumeLayout(false);

      }
      #endregion

      private void exitMenu_Click(object sender, System.EventArgs e) {
         Application.Exit();
      }

      private void hotKeyTree_BeforeCollapse(object sender, System.Windows.Forms.TreeViewCancelEventArgs e) {
         e.Cancel = true;
      }

      private void hotKeyTree_DoubleClick(object sender, System.EventArgs e) {
         TreeNode selected = hotKeyTree.SelectedNode;

         setUnsetHotKey(selected);
      }

      private void setUnsetHotKey(TreeNode selected) {
         // Don't go on if it's not a menu item
         if (selected.Parent == null) return;

         TrayMenuItem tmi = (TrayMenuItem)selected.Tag;

         HotKeyForm hkf;
         
         String id = String.Format("{0}/{1}", selected.Parent.Text, tmi.Name);
         // Determine if we've already got a key, and display it in the text box...
         SystemHotKey currentKey = (SystemHotKey)hotKeys[id];
         if (currentKey != null) {
            hkf = new HotKeyForm(currentKey.Key.Key, currentKey.Key.WindowsKey);
         } else {
            hkf= new HotKeyForm(Keys.None, false);
         }


         if (hkf.ShowDialog(this) == DialogResult.OK) {
            WhichHotKey whk = new WhichHotKey(hkf.KeyPressed, hkf.WinPressed);
            SystemHotKey shk;

            foreach (SystemHotKey sk in hotKeys.Values) {
               if (sk.Key == whk) {
                  MessageBox.Show("Key already registered! -> " + sk.Key.ToString());
                  return;
               }
            }

            if (hotKeys.ContainsKey(id) ) {
               shk = (SystemHotKey)hotKeys[id];
               shk.unsetHotKey();
               hotKeys.Remove(id);
            }

            if (whk.Key != Keys.None) {
               shk = new SystemHotKey(dummyWindow, whk, tmi.MenuEventHandler, tmi.Name);
               hotKeys.Add(id, shk);
               selected.Text = String.Format("{0} ({1})", tmi.Name, whk.ToString());
            } else {
               selected.Text = String.Format(tmi.Name);
            }
         }
      }

      private void programListView_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e) {
         ListViewItem currentItem = programListView.Items[e.Index];
         AppDomainInformation adi = (AppDomainInformation)currentItem.Tag;
         adi.IsActive = (e.NewValue == CheckState.Checked);

         if (adi.IsActive == false) {
            for (int i = 0; i < hotKeyTree.Nodes.Count; i++) {
               if (hotKeyTree.Nodes[i].Text == adi.Name) {
                  hotKeyTree.Nodes.RemoveAt(i);
                  i--;
               }
            }
         } else {
            addHotKeyList(adi);
         }
      }

      private void loadNewPlugin_Click(object sender, System.EventArgs e) {
			Debug.WriteLine("loadNewPlugin");
         FolderBrowserDialog fbd = new FolderBrowserDialog();
         fbd.RootFolder = System.Environment.SpecialFolder.MyComputer;
         fbd.ShowNewFolderButton = false;

         if (fbd.ShowDialog() == DialogResult.OK) {
            DirectoryInfo di = new DirectoryInfo(fbd.SelectedPath);
            FileInfo[] files = di.GetFiles("*.exe");
            if (files.Length == 1) {
               FileStream fs = new FileStream(files[0].FullName, FileMode.Open);
               byte[] fileData = new byte[fs.Length];
               fs.Read(fileData, 0, fileData.Length);
               fs.Close();
               Assembly assm = Assembly.Load(fileData);
               Type[] types = assm.GetTypes();
               foreach (Type t in types) {
                  Object[] attrs = t.GetCustomAttributes(typeof(TrayProgramAttribute), true);
                  if (attrs.Length > 0) {
                     TrayProgramAttribute attr = (TrayProgramAttribute)attrs[0];
                     String name = t.Name;
                     if (attr.ProgramName != null && attr.ProgramName != "") name = attr.ProgramName;
                     if (NewPlugin != null) {
                        NewPlugin(this, new NewPluginEventArgs(fbd.SelectedPath, name));
                     }
                     foreach (String domainName in domains.Keys) {
                        if (domainName.ToLower() == name.ToLower() ) {
                           addDomainInfo(name, (AppDomainInformation)domains[domainName]);
                        }
                     }
                     // All done, so break exit the routine
                     return;
                  }
               }
            }
            // If there were no files, or none had the right attribute we end up here with a warning
            MessageBox.Show("No plugin located");
         }
      }

      private void hotKeyTree_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
         TreeNode tn = hotKeyTree.SelectedNode;
         if (e.Button == MouseButtons.Right && tn != null) {
            ContextMenu cm = new ContextMenu(
               new MenuItem[1] { new MenuItem("Set/Unset Hot Key - " + tn.Text, new EventHandler(setUnsetHotkey_Click) ) } );
            cm.Show(hotKeyTree, new Point(e.X, e.Y));
         }
      }

      private void setUnsetHotkey_Click(Object sender, EventArgs e) {
         TreeNode tn = hotKeyTree.SelectedNode;

         setUnsetHotKey(tn);
      }


      /// <summary>
      /// Called when a new plugin is loaded into the system.
      /// </summary>
      internal event NewPluginEventHandler NewPlugin;

      internal event NewPluginDirectoryEventHandler NewPluginDirectory;

      private void menuHelpAbout_Click(object sender, EventArgs e) {
         MessageBox.Show("Tray Runner Framework & Associated Plugins Copyright 2005 by Chris Szurgot");
      }
   }      
}
