using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace TrayRunner {
   /// <summary>
   /// Helper class to show a HotKeyDialog. Wraps a HotKeySelector and an OK in a dialog box.
   /// </summary>
   public class HotKeyForm : System.Windows.Forms.Form {
      private HotKeySelector hotKey;
      private System.Windows.Forms.Button okButton;
      private System.Windows.Forms.Button clearButton;
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;

      public HotKeyForm() {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();
      }

      public HotKeyForm(Keys keyPressed, bool winPressed) : this() {
         hotKey.KeyPressed = keyPressed;
         hotKey.WinPressed = winPressed;
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

      /// <summary>
      /// Is the Windows part of the key combination? (Otherwise use Ctrl + Alt)
      /// </summary>
      public bool WinPressed {
         get { return hotKey.WinPressed; }
      }

      /// <summary>
      /// What key was pressed.
      /// </summary>
      public Keys KeyPressed {
         get { return hotKey.KeyPressed; }
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         this.hotKey = new TrayRunner.HotKeySelector();
         this.okButton = new System.Windows.Forms.Button();
         this.clearButton = new System.Windows.Forms.Button();
         this.SuspendLayout();
         // 
         // hotKey
         // 
         this.hotKey.KeyPressed = System.Windows.Forms.Keys.None;
         this.hotKey.Location = new System.Drawing.Point(8, 8);
         this.hotKey.Name = "hotKey";
         this.hotKey.Size = new System.Drawing.Size(280, 20);
         this.hotKey.TabIndex = 0;
         this.hotKey.Text = "";
         this.hotKey.WinPressed = false;
         // 
         // okButton
         // 
         this.okButton.DialogResult = System.Windows.Forms.DialogResult.OK;
         this.okButton.Location = new System.Drawing.Point(65, 32);
         this.okButton.Name = "okButton";
         this.okButton.TabIndex = 1;
         this.okButton.Text = "OK";
         // 
         // clearButton
         // 
         this.clearButton.Location = new System.Drawing.Point(153, 32);
         this.clearButton.Name = "clearButton";
         this.clearButton.TabIndex = 2;
         this.clearButton.Text = "Clear";
         this.clearButton.Click += new System.EventHandler(this.clearButton_Click);
         // 
         // HotKeyForm
         // 
         this.AcceptButton = this.okButton;
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(292, 62);
         this.Controls.Add(this.clearButton);
         this.Controls.Add(this.okButton);
         this.Controls.Add(this.hotKey);
         this.Name = "HotKeyForm";
         this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
         this.Text = "HotKeyForm";
         this.ResumeLayout(false);

      }
      #endregion

      private void clearButton_Click(object sender, System.EventArgs e) {
         hotKey.KeyPressed = Keys.None;
      }
   }
}
