using System;
using System.Windows.Forms;

namespace TrayRunner {
   /// <summary>
   /// Summary description for HotKeySelector.
   /// </summary>
   public class HotKeySelector : TextBox {
      public HotKeySelector() {
         this.KeyDown +=new KeyEventHandler(HotKeySelector_KeyDown);
         this.KeyUp += new KeyEventHandler(HotKeySelector_KeyUp);
         this.KeyPress += new KeyPressEventHandler(HotKeySelector_KeyPress);
      }

      // Private variable to determine if the WIndows Key is down at the time of
      // a valid key press.
      private bool isWinPressed = false;
      
      private bool _winPressed = false;
      /// <summary>
      /// Is the Windows part of the key combination? (Otherwise use Ctrl + Alt)
      /// </summary>
      public bool WinPressed {
         get { return _winPressed; }
         set { 
            _winPressed = value; 
            if (_keyPressed != Keys.None) {
               HotKeySelector_KeyDown(this, new KeyEventArgs(_keyPressed));
            }
         }
      }

      private Keys _keyPressed = Keys.None;
      /// <summary>
      /// What key was pressed
      /// </summary>
      public Keys KeyPressed {
         get { return _keyPressed; }
         set { 
            _keyPressed = value ;
            HotKeySelector_KeyDown(this, new KeyEventArgs(_keyPressed));
         }
      }

      private void HotKeySelector_KeyDown(Object sender, KeyEventArgs e) {
         if (e.KeyCode == Keys.RWin || e.KeyCode == Keys.LWin) {
            isWinPressed = true;
         }

         if ( (e.KeyValue >= (int)'A' && e.KeyValue <= (int)'Z') ||
            (e.KeyValue >= (int) 'a' && e.KeyValue <= (int)'z') ||
            (e.KeyValue >= (int)'0' && e.KeyValue <= (int)'9') ||
            (e.KeyValue >= (int)Keys.F2 && e.KeyValue <= (int)Keys.F24) ) {
            _keyPressed = e.KeyCode;
            if (isWinPressed == true) {
               this.Text = "Win + " + e.KeyCode;
               _winPressed = true;
            } else {
               this.Text = "Ctrl + Alt + " + e.KeyCode;
               _winPressed = false;
            }
         } else {
            this.Text = "None";
            _keyPressed = Keys.None;
            _winPressed = false;
         }

         e.Handled = true;
         // Move the selection back to the beginning
         this.Select(0,0);
      }

      private void HotKeySelector_KeyPress(Object sender, KeyPressEventArgs e) {
         e.Handled = true;
      }

      private void HotKeySelector_KeyUp(Object sender, KeyEventArgs e) {
         e.Handled = true;

         if (e.KeyCode == Keys.LWin || e.KeyCode == Keys.RWin) {
            isWinPressed = false;
         }
      }
   }
}
