/* Code liberally lifted from a sample on CodeProject.com
 * http://www.codeproject.com/useritems/systemhotkey.asp
 * 
 * Pleze visit this sample to see the original code.
 */

using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using TrayRunnerFramework;
using TrayRunner.Win32;

namespace TrayRunner  {
   /// <summary>
   /// Handles a System Hotkey.
   /// </summary>
   [Serializable]
   public class SystemHotKey : IDisposable {
      
      protected DummyWindowWithEvent m_Window;

      int keyId;
      
      public event System.EventHandler Pressed;
      bool isRegistered;

      WhichHotKey _key;
      /// <summary>
      /// The actual Keystroke for this hot key.
      /// </summary>
      public WhichHotKey Key {
         get { return _key; }
      }

      // We only want one instance of the message handler, so we keep it handle to add/remove from the delegate
      private MessageEventHandler _messageHandler;

      bool _active;
      /// <summary>
      /// Gets or sets whether the current hot key is active
      /// </summary>
      public bool Active {
         get { return _active; }
         set {
            if (_active != value) {
               if (value == true) {
                  RegisterHotkey((char)_key.Key, _key.WindowsKey);
                  m_Window.ProcessMessage += _messageHandler;
               } else {
                  if (isRegistered) {
//                     if (UnregisterHotkey())
//                        System.Diagnostics.Debug.WriteLine("Unreg: OK");
							UnregisterHotkey();
                     m_Window.ProcessMessage -= _messageHandler;
                  }
               }
               _active = value;
            }
         }
      }

		private String _eventName;

      public SystemHotKey(DummyWindowWithEvent window, WhichHotKey key, EventHandler keyPressed, String eventName) {
			_eventName = eventName;

         _messageHandler = new MessageEventHandler(MessageEvent);
         
         m_Window = window;
         this._key = key;
         this.Pressed += keyPressed;
         Active = true;
      }

      public void Dispose() {
         if (isRegistered) {
				UnregisterHotkey();
         }
      }

      protected void MessageEvent(object sender,ref Message m,ref bool Handled) {
         long lParam = m.LParam.ToInt64();
         long hash = _key.GetHashCode();

         if ((m.Msg==(int)Msgs.WM_HOTKEY)&& lParam == hash) {
            Handled=true;
            if (Pressed != null) Pressed(_eventName, EventArgs.Empty);
         }
      }
	
      protected bool UnregisterHotkey() {
         bool retVal = User32.UnregisterHotKey(m_Window.Handle, keyId);
         Kernel32.GlobalDeleteAtom(keyId);
         //unregister hotkey
         isRegistered = false;

         return retVal;
      }

      protected bool RegisterHotkey(char key, bool winPressed) {
         //register hotkey
         int mod=0;
         if (winPressed == true) {
            mod = (int)Modifiers.MOD_WIN;
         } else {
            mod = (int)Modifiers.MOD_ALT | (int)Modifiers.MOD_CONTROL;
         }
         Keys k2 = Keys.None;
         keyId = Kernel32.GlobalAddAtom(key.ToString() );
         isRegistered = true;
         return User32.RegisterHotKey(m_Window.Handle,keyId,(int)mod,((int)key)-((int)k2));
      }

      public void unsetHotKey() {
         Active = false;
         UnregisterHotkey();         
      }
   }
}
