// TODO Show icon, *THEN* load up AppDomains

using System;
using System.Windows.Forms;
using System.Drawing;

using System.Reflection;
using System.Diagnostics;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Runtime.Remoting;

using TrayRunnerFramework;

namespace TrayRunner {
   /// <summary>
   /// Context class that shows the icon and hosts all the AppDomains
   /// </summary>
   public class TrayContextClass : System.Windows.Forms.ApplicationContext {
      System.Windows.Forms.NotifyIcon notifyIcon;
      System.Windows.Forms.ContextMenu contextMenu;
      Icon icon;

      Timer iconTimer = new Timer();

      // Stores the actual app domains
      Hashtable domains = new Hashtable();
      // Stores any hotkeys in local memory
		Hashtable hotKeys = new Hashtable();
		// Stored the app domain filename
		ArrayList domainFiles = new ArrayList();

      TrayRunner.Win32.DummyWindowWithEvent dummyWindow;

      ConfigForm cf = null;

//      Hashtable pluginDirectories = new Hashtable();

      ArrayList loadedPlugins = new ArrayList();

      Properties props;

		public TrayContextClass(String[] args) {
			this.ThreadExit +=new EventHandler(TrayContextClass_ThreadExit);

			Assembly assm = Assembly.GetExecutingAssembly();
			FileInfo fi = new FileInfo(assm.Location);

			// Let's build and popup the icon so the user can see the program is active
			notifyIcon = new System.Windows.Forms.NotifyIcon();
			icon = new System.Drawing.Icon(assm.GetManifestResourceStream("TrayRunner.desktop.ico"));
			notifyIcon.Icon = icon;
			notifyIcon.DoubleClick += new EventHandler(onConfig);
         

			contextMenu = new ContextMenu(new MenuItem[] { 
																			new MenuItem("-"), 
                                                         new MenuItem("Show Key List", new EventHandler(showKeyList)), 
																			new MenuItem("Config", new EventHandler(onConfig)),
																			new MenuItem("Close", new EventHandler(onClose)) } );
			notifyIcon.ContextMenu = contextMenu;

			notifyIcon.Visible = true;

			props = (Properties)TrayRunnerFramework.SaveProperties.LoadObject("TrayRunner.config");
			if (props == null) {
				props = new Properties();
			}

         if (args.Length > 1) {
            props.PluginDirectories.Add(args[1]);
         }

			if (props.PluginDirectories.Count == 0) {
				// TODO Find a way to set one here
				MessageBox.Show("Please Set a plugins directory");
			}

			Process p = Process.GetCurrentProcess();
			Process[] all = Process.GetProcessesByName(p.ProcessName);

			if (all.Length > 1) {
				MessageBox.Show("Already Running! Please check the system tray.");
				Environment.Exit(1);
			}

			dummyWindow = new TrayRunner.Win32.DummyWindowWithEvent();

			WhichHotKey whk = new WhichHotKey("Ctrl + Atrl + F1");
			SystemHotKey shk = new SystemHotKey(dummyWindow, whk, new EventHandler(showKeyList), "Hot Key List");
			hotKeys.Add("Show Key List", shk);
			
			TrayRunnerFramework.PluginInformation pl = new TrayRunnerFramework.PluginInformation();

			foreach (String pluginName in props.Plugins) {
				try {
					PluginInformation pi = 
						(PluginInformation)TrayRunnerFramework.SaveProperties.LoadObject(pluginName + ".plugin");

					createAppDomain(pi.AssemblyFile, pi);
				} catch (Exception ex) {
					MessageBox.Show("Plugin Exception:" + ex);
				}
			}

			// Ok, are there any unloaded domains at this point?
			foreach (String dir in props.PluginDirectories) {
				discoverNewPlugins(dir);
			}

			iconTimer.Interval = 1000 * 60;
			iconTimer.Tick += new EventHandler(iconTimer_Tick);
			iconTimer.Enabled = true;
		}

		private void showKeyList(Object sender, EventArgs e) {
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append("Current Hot Keys\n");
			foreach (String key in hotKeys.Keys) {
				TrayRunner.SystemHotKey shk = (TrayRunner.SystemHotKey)hotKeys[key];
				
				sb.AppendFormat("KEY: {0} -> {1}\n", key, shk.Key);
			}
			MessageBox.Show(sb.ToString(), "Current Hot Keys", MessageBoxButtons.OK, MessageBoxIcon.Information, 
				MessageBoxDefaultButton.Button1, MessageBoxOptions.ServiceNotification);
		}

      private void shutdown() {
			// TODO Save hotkeys first since they go away as the AppDomain is shutdown
			props.HotKeys.Clear();
			foreach (String name in hotKeys.Keys) {
				if (name != "Show Key List") {
					SystemHotKey shk = (SystemHotKey)hotKeys[name];
					props.HotKeys.Add(name, shk.Key.ToString());
				}
			}

         props.Plugins.Clear();
			foreach (AppDomainInformation adi in domains.Values) {
            PluginInformation pi = new PluginInformation();
            
            pi.AssemblyFile = adi.AssemblyFile;
            pi.AssemblyName = adi.AssemblyName;
            pi.Name = adi.Name;
            pi.TypeName = adi.TypeName;
            pi.IsActive = adi.IsActive;

            if (adi.Plugin.PluginIcon != null) {
               pi.PluginIcon = adi.Plugin.PluginIcon;
            }

            TrayRunnerFramework.SaveProperties.SaveObject(pi, pi.Name + ".plugin");
				props.Plugins.Add(pi.Name);

				// Now that we've saved the state, shutdown the domain so it cleans itself up
				adi.IsActive = false;
         }

			TrayRunnerFramework.SaveProperties.SaveObject(props, "TrayRunner.config");
		}

      private void onClose(Object sender, EventArgs e) {
         Application.Exit();
      }

		[STAThread]
		[LoaderOptimization(LoaderOptimization.MultiDomain)]
		public static void Main(String[] args) {
			if (args.Length > 0 && args[0] == "-restart") {
				// Wait 1 second to give the old program a chance to shut down
				System.Threading.Thread.Sleep(1000);
			}
			Application.Run(new TrayContextClass(args) );
		}

      /// <summary>
      /// Checks a plugin directory for any new plugins
      /// </summary>
      /// <param name="dirName"></param>
      internal void discoverNewPlugins(String dirName) {
         DirectoryInfo currentDir = new DirectoryInfo(dirName.ToString());
         if (currentDir.Exists) {
				foreach (FileInfo fi in currentDir.GetFiles() ) {
					if ( (fi.Extension == ".exe" || fi.Extension == ".dll") && fi.Name != "TrayRunnerFramework.dll") {
                  try {
                     if (!domainFiles.Contains(fi.FullName)) {
                        Hashtable plugins;
                        AssemblyName fullAssmName;
                        ITrayProgramLoader.ProbeAssembly(fi.Directory.FullName, fi.Name, out fullAssmName, out plugins);

                        foreach (PluginInformation pi in plugins.Values) {
                           createAppDomain(fi, pi);
                        }
                     }
                  } catch (Exception ex) {
                     System.Diagnostics.Trace.WriteLine("EX:" + ex);
                  }
					}
				}
         }
      }

      // Creates the app domain
      private void createAppDomain(FileInfo file, PluginInformation plugin) {
         try {
            AppDomainInformation adi = new AppDomainInformation(file, plugin);
            domains.Add(plugin.Name, adi);

				domainFiles.Add(file.FullName);

            if (adi.IsActive == true) {
               loadHotKeysFromAppDomain(adi);
               loadTrayMenuItemsFromAppDomain(adi);
            }

            adi.ActivateAppDomain += new EventHandler(adi_ActivateAppDomain);
            adi.DeactivateAppDomain += new EventHandler(adi_DeactivateAppDomain);
         } catch (FileNotFoundException fnfex) { // Disregard. Means the files are names wrong
				Debug.WriteLine("EX:" + fnfex);
         } catch (Exception ex) {
				System.Diagnostics.Trace.WriteLine(String.Format("EX loading {0} -> {1}\n{2}", file.DirectoryName, plugin.Name, ex.ToString() ) );
         }
      }

      private void loadHotKeysFromAppDomain(AppDomainInformation adi) {
			foreach (String hotKeyName in props.HotKeys.Keys) {
				String[] strings = hotKeyName.Split('/');
				if (strings[0] == adi.Name) {
					for (int i = 0; i < adi.TrayMenu.Length; i++) {
						if (adi.TrayMenu[i].Name == strings[1]) {
							WhichHotKey whk = new WhichHotKey((String)props.HotKeys[hotKeyName]);
							SystemHotKey shk = new SystemHotKey(dummyWindow, whk, adi.TrayMenu[i].MenuEventHandler,
								adi.TrayMenu[i].Name);

							hotKeys.Add(hotKeyName, shk);
						}
					}
				}
			}
      }

      private void unloadHotKeysFromAppDomain(AppDomainInformation adi) {
         ArrayList removed = new ArrayList();
         foreach (String key in hotKeys.Keys) {
             if (key.StartsWith(adi.Name) ) {
               SystemHotKey shk = (SystemHotKey)hotKeys[key];
               shk.unsetHotKey();
               removed.Add(key);
            }
         }

         foreach (String key in removed) {
            hotKeys.Remove(key);
         }
      }

      private void loadTrayMenuItemsFromAppDomain(AppDomainInformation adi) {
         MenuItem newMenu = new MenuItem(adi.Name);
         contextMenu.MenuItems.Add(0, newMenu);

         if (adi.TrayMenu == null || adi.TrayMenu.Length == 0) {
            newMenu.Click += new EventHandler(newMenu_Click);
         } else {
            for (int i = 0; i < adi.TrayMenu.Length; i++) {
               MenuItem mi = new MenuItem(adi.TrayMenu[i].Name, new EventHandler(adi.TrayMenu[i].MenuEventHandler));
               newMenu.MenuItems.Add(mi);
            }
         }
      }

      private void unloadTrayMenuItemsFromAppDomain(AppDomainInformation adi) {
         for (int i = 0; i < contextMenu.MenuItems.Count; i++) {
            if (contextMenu.MenuItems[i].Text == adi.Name) {
               contextMenu.MenuItems.RemoveAt(i);
               i--;
            }
         }
      }
         

      private void newMenu_Click(Object sender, EventArgs e) {
         MessageBox.Show("No menu items available for this class");
      }

      private void onConfig(object sender, EventArgs e) {
         if (cf == null || !cf.Visible) {
            cf = new ConfigForm(dummyWindow, domains, hotKeys, props.PluginDirectories);
            cf.NewPlugin += new NewPluginEventHandler(cf_NewPlugin);
            cf.NewPluginDirectory += new NewPluginDirectoryEventHandler(cf_NewPluginDirectory);
            cf.ShowDialog();
         } else {
            cf.BringToFront();
         }
      }

      private void TrayContextClass_ThreadExit(object sender, EventArgs e) {
         shutdown();
         notifyIcon.Visible = false;
      }

// TODO Re-institute new plugins
      private void cf_NewPlugin(Object sender, NewPluginEventArgs e) {
System.Diagnostics.Trace.WriteLine("New Plugin");
         
         //         createAppDomain(e.ClassName, e.PluginLocation, true);
      }

      private void cf_NewPluginDirectory(Object sender, EventArgs e) {
      }

      private void cf_NewPluginDirectory(Object sender, NewPluginDirectoryEventArgs e) {
         discoverNewPlugins(e.Directory);
      }

      private void adi_ActivateAppDomain(object sender, EventArgs e) {
         AppDomainInformation adi = (AppDomainInformation)sender;
         loadTrayMenuItemsFromAppDomain(adi);
         loadHotKeysFromAppDomain(adi);
      }

      private void adi_DeactivateAppDomain(object sender, EventArgs e) {
         AppDomainInformation adi = (AppDomainInformation)sender;
         unloadHotKeysFromAppDomain(adi);
         unloadTrayMenuItemsFromAppDomain(adi);
		}

      private void iconTimer_Tick(object sender, EventArgs e) {
         notifyIcon.Icon = icon;      
      }
   }
}
