 using System;
using System.Reflection;
using System.Diagnostics;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace TrayRunnerFramework {
   /// <summary>
   /// The main function. Uses a ContextClass so it can be a Windows Program without a GUI 
   /// (just a tray icon) This version currently has minimal hooks, and I hope to make them even
   /// less in future versions (for other tray programs)
   /// </summary>
   /// <remarks>
   ///      private static void Main() {
   ///  System.Windows.Forms.Application.Run(new TrayRunnerFramework.ContextClass() );
   ///}
   ///</remarks>
   public class ContextClass : System.Windows.Forms.ApplicationContext {
      System.Windows.Forms.NotifyIcon icon;
      System.Windows.Forms.ContextMenu contextMenu = null;

      ITrayProgram program;

      public ContextMenu IconMenu {
         get { return contextMenu; }
      }

      public ContextClass() {
         AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);

         icon = new System.Windows.Forms.NotifyIcon();

         Assembly assm = Assembly.GetCallingAssembly();

         Process p = Process.GetCurrentProcess();
         Process[] all = Process.GetProcessesByName(p.ProcessName);

         if (all.Length > 1) {
            MessageBox.Show("Already Running! Please check the system tray.");
            Environment.Exit(1);
         }

         bool typeFound = false;
         String iconFileName = null;
         foreach (Type t in assm.GetTypes() ) {
            Object[] attrs = t.GetCustomAttributes(typeof(TrayProgramAttribute), true);
            if (attrs.Length > 0) {
               typeFound = true;
               program = (ITrayProgram)Activator.CreateInstance(t, new Object[0]);
               iconFileName = ((TrayProgramAttribute)attrs[0]).IconFileName;
            }
         }
         if (typeFound == false){ 
            MessageBox.Show("No Type To Load");
            Environment.Exit(1);
         }

         contextMenu = new ContextMenu();
         foreach (TrayRunnerFramework.TrayMenuItem mi in program.TrayMenu) {
            contextMenu.MenuItems.Add(new MenuItem(mi.Name, mi.MenuEventHandler));
         }
         contextMenu.MenuItems.Add(new MenuItem("-"));
         contextMenu.MenuItems.Add( new MenuItem("Close", new EventHandler(onClose)) );
         icon.ContextMenu = contextMenu;

         // The last thing we do is look for an Icon.
         bool foundIcon = false;
         try {
            System.IO.Stream s = assm.GetManifestResourceStream(iconFileName);
            if (s != null) {
               icon.Icon = new System.Drawing.Icon(s);
               foundIcon = true;
            }
         } catch (Exception ex) {
            MessageBox.Show("Icon:" + ex);
         }
         if (!foundIcon) {
            iconFileName = iconFileName.ToLower();
            foreach (String name in assm.GetManifestResourceNames() ) {
               if (name.ToLower().EndsWith(iconFileName) ) {
                  icon.Icon = new System.Drawing.Icon(assm.GetManifestResourceStream(name));
                  foundIcon = true;
                  break;
               }
            }
         }
         if (!foundIcon) {
            MessageBox.Show("No Icon to load. Can't load in tray!");
            Environment.Exit(1);
         }
         icon.Visible = true;



         System.AppDomain.CurrentDomain.ProcessExit += new EventHandler(onProcessExit);
         System.AppDomain.CurrentDomain.DomainUnload += new EventHandler(onProcessExit);
      }

      // Turn off the icon, and save the data.
      // "It's my job to put it all in order, now, and lock the place behind me as I leave."
      private void onProcessExit(Object sender, EventArgs e) {
         icon.Visible = false;
         program.OnShutdown();
      }

      /// <summary>
      /// Main function. If we don't have any paramters, start normally, otherwise assume 
      /// the arguments are file names and draw and change the background image
      /// </summary>
      /// <param name="args"></param>
      public static void Main() {
         AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
         Application.Run(new ContextClass() );
      }

      private static void CurrentDomain_UnhandledException(Object sender, UnhandledExceptionEventArgs e) {
         MessageBox.Show("Exception:" + e.ExceptionObject);
      }

      // Menu event. Just shut down the program.
      private void onClose(Object sender, EventArgs e) {
         Application.Exit();
      }
   }
}
