using System;
using System.Windows.Forms;

namespace TrayRunnerFramework {
   /// <summary>
   /// Assign this attribute to the class that will loaded as the ContextClass or into the new appdomain
   /// </summary>
   public class TrayProgramAttribute : System.Attribute {
      private String _programName = null;
      /// <summary>
      /// Visible Name of the program. If not set, the name will be the subdirectory
      /// </summary>
      public String ProgramName {
         get { return _programName; }
      }
      
      private String _iconFileName;
      /// <summary>
      /// Manifest FileName of the icon for the program.
      /// </summary>
      public String IconFileName {
         get { return _iconFileName; }
         set { _iconFileName = value; }
      }

      private String _description;
      public String Description {
         get { return _description; }
         set { _description = value; }
      }
      
      /// <summary>
      /// Use to set the program name
      /// </summary>
      /// <param name="programName">Program name that will be displayed</param>
      public TrayProgramAttribute(String programName, String iconFileName) {
         _programName = programName;
         _iconFileName = iconFileName;
      }
   }

   public interface ITrayProgram {
      /// <summary>
      /// Create an Array of TrayMenuItems which will be passed to the main program or the context class
      /// if standalone.
      /// </summary>
      TrayMenuItem[] TrayMenu {
         get ;
      }

      /// <summary>
      /// Called when program domain is being unloaded. Allow the class to perform any final shutdown.
      /// </summary>
      void OnShutdown();
   }

   [Serializable]
   public class TrayMenuItem {
      private String _name;
      public String Name {
         get { return _name; }
      }

      private EventHandler _menuEventHandler;
      public EventHandler MenuEventHandler {
         get { return _menuEventHandler; }
      }

      public TrayMenuItem(String name, EventHandler menuEventHandler) {
         _name = name;
         _menuEventHandler = menuEventHandler;
      }
   }
}